/*

Copyright (c) Bridgetek Pte Ltd

THIS SOFTWARE IS PROVIDED BY BRIDGETEK PTE LTD "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
BRIDGETEK PTE LTD BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES LOSS OF USE, DATA, OR PROFITS OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

BRIDGETEK DRIVERS MAY BE USED ONLY IN CONJUNCTION WITH PRODUCTS BASED ON BRIDGETEK PARTS.

BRIDGETEK DRIVERS MAY BE DISTRIBUTED IN ANY FORM AS LONG AS LICENSE INFORMATION IS NOT MODIFIED.

IF A CUSTOM VENDOR ID AND/OR PRODUCT ID OR DESCRIPTION STRING ARE USED, IT IS THE
RESPONSIBILITY OF THE PRODUCT MANUFACTURER TO MAINTAIN ANY CHANGES AND SUBSEQUENT WHQL
RE-CERTIFICATION AS A RESULT OF MAKING THESE CHANGES.

Abstract:

Application to demonstrate function of EVE.

Author : Bridgetek

Revision History:
0.1 - date 2017.03.24 - Initial version
*/
#include "Platform.h"
#include "App_Common.h"


#if ( defined(FT900_PLATFORM) || defined(FT93X_PLATFORM) )
#include "ff.h"

#endif

/*************************************************************************************************
*                      These functions work with FT8XX GPU buffer  
**************************************************************************************************/
uint32_t CmdBuffer_Index;
volatile uint32_t DlBuffer_Index;

#ifdef BUFFER_OPTIMIZATION
uint8_t  DlBuffer[DL_SIZE];
uint8_t  CmdBuffer[CMD_FIFO_SIZE];
#endif

void App_WrCoCmd_Buffer(Gpu_Hal_Context_t *phost,uint32_t cmd)
{
#ifdef  BUFFER_OPTIMIZATION
    /* Copy the command instruction into buffer */
    uint32_t *pBuffcmd;
	/* Prevent buffer overflow */
	if (CmdBuffer_Index >= CMD_FIFO_SIZE)
	{
		//printf("CmdBuffer overflow\n");

		if (CmdBuffer_Index > 0) {
			Gpu_Hal_WrCmdBuf(phost, CmdBuffer, CmdBuffer_Index);//This blocking state may be infinite due to Display list overflow
		}
		CmdBuffer_Index = 0;
	}

	pBuffcmd = (uint32_t*)&CmdBuffer[CmdBuffer_Index];
	*pBuffcmd = cmd;


#endif

#if ( defined(FT900_PLATFORM) || defined(FT93X_PLATFORM) || defined(ARDUINO_PLATFORM) )
    Gpu_Hal_WrCmd32(phost,cmd);
#endif
    /* Increment the command index */
    CmdBuffer_Index += CMD_SIZE;
}

void App_WrDl_Buffer(Gpu_Hal_Context_t *phost,uint32_t cmd)
{
#ifdef BUFFER_OPTIMIZATION  
    /* Copy the command instruction into buffer */
    uint32_t *pBuffcmd;
	/* Prevent buffer overflow */
	if (DlBuffer_Index < DL_SIZE)
	{
		pBuffcmd = (uint32_t*)&DlBuffer[DlBuffer_Index];
		*pBuffcmd = cmd;
	}
	else
	{
		printf("DlBuffer overflow\n");
	}

#endif

#if ( defined(FT900_PLATFORM) || defined(FT93X_PLATFORM) || defined(ARDUINO_PLATFORM) )
    Gpu_Hal_Wr32(phost,(RAM_DL+DlBuffer_Index),cmd);
#endif
    /* Increment the command index */
    DlBuffer_Index += CMD_SIZE;  
}

void App_WrCoStr_Buffer(Gpu_Hal_Context_t *phost,const char8_t *s)
{
#ifdef  BUFFER_OPTIMIZATION  
    uint16_t length = 0;

	if (CmdBuffer_Index >= CMD_FIFO_SIZE)
	{
		printf("CmdBuffer overflow\n");

		if (CmdBuffer_Index > 0) {
			Gpu_Hal_WrCmdBuf(phost, CmdBuffer, CmdBuffer_Index);//This blocking state may be infinite due to Display list overflow
		}
		CmdBuffer_Index = 0;
	}


	length = strlen(s) + 1;//last for the null termination

    strcpy((char*)&CmdBuffer[CmdBuffer_Index],s);  

    /* increment the length and align it by 4 bytes */
    CmdBuffer_Index += ((length + 3) & ~3);  
#endif  
}

void App_Flush_DL_Buffer(Gpu_Hal_Context_t *phost)
{
#ifdef  BUFFER_OPTIMIZATION
    if (DlBuffer_Index> 0)
    Gpu_Hal_WrMem(phost,RAM_DL,DlBuffer,DlBuffer_Index);
#endif
    DlBuffer_Index = 0;

}

void App_Flush_Co_Buffer(Gpu_Hal_Context_t *phost)
{
#ifdef  BUFFER_OPTIMIZATION    
    if (CmdBuffer_Index > 0)
    Gpu_Hal_WrCmdBuf(phost,CmdBuffer,CmdBuffer_Index);
#endif     
    CmdBuffer_Index = 0;
}

void App_Flush_Co_Buffer_nowait(Gpu_Hal_Context_t *phost)
{
#ifdef  BUFFER_OPTIMIZATION    
	if (CmdBuffer_Index > 0)
		Gpu_Hal_WrCmdBuf_nowait(phost, CmdBuffer, CmdBuffer_Index);
#endif     
	CmdBuffer_Index = 0;
}

void App_Set_DlBuffer_Index(uint32_t index)
{
    DlBuffer_Index = index;
}

void App_Set_CmdBuffer_Index(uint32_t index)
{
    CmdBuffer_Index = index;
}
/*************************************************************************************************
*                      Application Utilities
**************************************************************************************************/

static uint8_t sk=0;
uint8_t App_Read_Tag(Gpu_Hal_Context_t *phost)
{
    static uint8_t Read_tag=0,temp_tag=0,ret_tag=0;
    Read_tag = Gpu_Hal_Rd8(phost,REG_TOUCH_TAG);
    ret_tag = 0;
    if(Read_tag!=0)                                // Allow if the Key is released
    {
        if(temp_tag!=Read_tag)
        {
            temp_tag = Read_tag;
            sk = Read_tag;                                        // Load the Read tag to temp variable
        }
    }
    else
    {
        if(temp_tag!=0)
        {
            ret_tag = temp_tag;
            temp_tag = 0; //The event will be processed. Clear the tag
        }
        sk = 0;
    }
    return ret_tag;
}

uint8_t App_Touch_Update(Gpu_Hal_Context_t *phost, uint8_t *currTag, uint16_t *x, uint16_t *y)
{
	static uint8_t Read_tag = 0, temp_tag = 0, ret_tag = 0;
	uint32_t touch;
	Read_tag = Gpu_Hal_Rd8(phost, REG_TOUCH_TAG);
	ret_tag = 0;
	if (Read_tag != 0)                                // Allow if the Key is released
	{
		if (temp_tag != Read_tag)
		{
			temp_tag = Read_tag;
			sk = Read_tag;                                        // Load the Read tag to temp variable
		}
	}
	else
	{
		if (temp_tag != 0)
		{
			ret_tag = temp_tag;
			temp_tag = 0; //The event will be processed. Clear the tag
		}
		sk = 0;
	}
	*currTag = Read_tag;
	touch = Gpu_Hal_Rd32(phost, REG_TOUCH_SCREEN_XY); 
	*x = (uint16_t)(touch >> 16);
	*y = (uint16_t)(touch & 0xFFFF);
	return ret_tag;
}
void App_Play_Sound(Gpu_Hal_Context_t *phost, uint8_t sound,uint8_t vol,uint8_t midi)
{
    uint16_t val = (midi << 8) | sound; 
    Gpu_Hal_Wr8(phost,REG_VOL_SOUND,vol);
    Gpu_Hal_Wr16(phost,REG_SOUND,val);
    Gpu_Hal_Wr8(phost,REG_PLAY,1); 
}


void App_Calibrate_Screen(Gpu_Hal_Context_t *phost)
{
    Gpu_CoCmd_Dlstart(phost);
    App_WrCoCmd_Buffer(phost,CLEAR(1,1,1));
    App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
    Gpu_CoCmd_Text(phost,DispWidth/2,DispHeight/2,26,OPT_CENTERX|OPT_CENTERY,"Please tap on a dot");

    /* Calibration */
    Gpu_CoCmd_Calibrate(phost,0);
    App_Flush_Co_Buffer(phost);
    Gpu_Hal_WaitCmdfifo_empty(phost);
}


/* This image contains cells: 32x32
Cell 0: Home
Cell 1: Star
Cell 2: Down triangle
Cell 3: Up triangle
Cell 4: Right triangle (Play button icon)
Size: 460 bytes */
PROGMEM prog_uchar8_t home_start_icon[SIZE_HOME_START_ICON] = {0x78,0x9C,0xE5,0x94,0xBF,0x4E,0xC2,0x40,0x1C,0xC7,0x7F,0x2D,0x04,0x8B,0x20,0x45,0x76,0x14,0x67,0xA3,0xF1,0x0D,0x64,0x75,0xD2,0xD5,0x09,0x27,0x17,0x13,0xE1,0x0D,0xE4,0x0D,0x78,0x04,0x98,0x5D,0x30,0x26,0x0E,0x4A,0xA2,0x3E,0x82,0x0E,0x8E,0x82,0xC1,0x38,0x62,0x51,0x0C,0x0A,0x42,0x7F,0xDE,0xB5,0x77,0xB4,0x77,0x17,0x28,0x21,0x26,0x46,0xFD,0x26,0xCD,0xE5,0xD3,0x7C,0xFB,0xBB,0xFB,0xFD,0xB9,0x02,0xCC,0xA4,0xE8,0x99,0x80,0x61,0xC4,0x8A,0x9F,0xCB,0x6F,0x31,0x3B,0xE3,0x61,0x7A,0x98,0x84,0x7C,0x37,0xF6,0xFC,0xC8,0xDD,0x45,0x00,0xDD,0xBA,0xC4,0x77,0xE6,0xEE,0x40,0xEC,0x0E,0xE6,0x91,0xF1,0xD2,0x00,0x42,0x34,0x5E,0xCE,0xE5,0x08,0x16,0xA0,0x84,0x68,0x67,0xB4,0x86,0xC3,0xD5,0x26,0x2C,0x20,0x51,0x17,0xA2,0xB8,0x03,0xB0,0xFE,0x49,0xDD,0x54,0x15,0xD8,0xEE,0x73,0x37,0x95,0x9D,0xD4,0x1A,0xB7,0xA5,0x26,0xC4,0x91,0xA9,0x0B,0x06,0xEE,0x72,0xB7,0xFB,0xC5,0x16,0x80,0xE9,0xF1,0x07,0x8D,0x3F,0x15,0x5F,0x1C,0x0B,0xFC,0x0A,0x90,0xF0,0xF3,0x09,0xA9,0x90,0xC4,0xC6,0x37,0xB0,0x93,0xBF,0xE1,0x71,0xDB,0xA9,0xD7,0x41,0xAD,0x46,0xEA,0x19,0xA9,0xD5,0xCE,0x93,0xB3,0x35,0x73,0x0A,0x69,0x59,0x91,0xC3,0x0F,0x22,0x1B,0x1D,0x91,0x13,0x3D,0x91,0x73,0x43,0xF1,0x6C,0x55,0xDA,0x3A,0x4F,0xBA,0x25,0xCE,0x4F,0x04,0xF1,0xC5,0xCF,0x71,0xDA,0x3C,0xD7,0xB9,0xB2,0x48,0xB4,0x89,0x38,0x20,0x4B,0x2A,0x95,0x0C,0xD5,0xEF,0x5B,0xAD,0x96,0x45,0x8A,0x41,0x96,0x7A,0x1F,0x60,0x0D,0x7D,0x22,0x75,0x82,0x2B,0x0F,0xFB,0xCE,0x51,0x3D,0x2E,0x3A,0x21,0xF3,0x1C,0xD9,0x38,0x86,0x2C,0xC6,0x05,0xB6,0x7B,0x9A,0x8F,0x0F,0x97,0x1B,0x72,0x6F,0x1C,0xEB,0xAE,0xFF,0xDA,0x97,0x0D,0xBA,0x43,0x32,0xCA,0x66,0x34,0x3D,0x54,0xCB,0x24,0x9B,0x43,0xF2,0x70,0x3E,0x42,0xBB,0xA0,0x95,0x11,0x37,0x46,0xE1,0x4F,0x49,0xC5,0x1B,0xFC,0x3C,0x3A,0x3E,0xD1,0x65,0x0E,0x6F,0x58,0xF8,0x9E,0x5B,0xDB,0x55,0xB6,0x41,0x34,0xCB,0xBE,0xDB,0x87,0x5F,0xA9,0xD1,0x85,0x6B,0xB3,0x17,0x9C,0x61,0x0C,0x9B,0xA2,0x5D,0x61,0x10,0xED,0x2A,0x9B,0xA2,0x5D,0x61,0x10,0xED,0x2A,0x9B,0xA2,0x5D,0x61,0x10,0xED,0x2A,0x9B,0xED,0xC9,0xFC,0xDF,0x14,0x54,0x8F,0x80,0x7A,0x06,0xF5,0x23,0xA0,0x9F,0x41,0xF3,0x10,0x30,0x4F,0x41,0xF3,0x18,0x30,0xCF,0xCA,0xFC,0xFF,0x35,0xC9,0x79,0xC9,0x89,0xFA,0x33,0xD7,0x1D,0xF6,0x5E,0x84,0x5C,0x56,0x6E,0xA7,0xDA,0x1E,0xF9,0xFA,0xAB,0xF5,0x97,0xFF,0x2F,0xED,0x89,0x7E,0x29,0x9E,0xB4,0x9F,0x74,0x1E,0x69,0xDA,0xA4,0x9F,0x81,0x94,0xEF,0x4F,0xF6,0xF9,0x0B,0xF4,0x65,0x51,0x08};

void App_Show_WelcomeScreen(Gpu_Hal_Context_t *phost, char* info[])
{
    //CmdBuffer_Index = 0;

    Gpu_Hal_WrCmd32(phost,CMD_INFLATE);
    Gpu_Hal_WrCmd32(phost,START_ICON_ADDR);
#if defined(MSVC_PLATFORM) || defined(MSVC_FT800EMU)
    Gpu_Hal_WrCmdBuf(phost,home_start_icon,sizeof(home_start_icon)); //Load from RAM
#else
    Gpu_Hal_WrCmdBufFromFlash(phost,home_start_icon,sizeof(home_start_icon)); //Load from Flash
#endif

    Gpu_CoCmd_Dlstart(phost);        // start
    App_WrCoCmd_Buffer(phost,CLEAR(1,1,1));
    App_WrCoCmd_Buffer(phost,COLOR_RGB(255, 255, 255));

    App_WrCoCmd_Buffer(phost,BITMAP_HANDLE(START_ICON_HANDLE));    // handle for background stars
    App_WrCoCmd_Buffer(phost,BITMAP_SOURCE(START_ICON_ADDR));      // Starting address in gram
    App_WrCoCmd_Buffer(phost,BITMAP_LAYOUT(L4, 16, 32));  // format
    App_WrCoCmd_Buffer(phost,BITMAP_SIZE(NEAREST, BORDER, BORDER, 32, 32  ));

    App_WrCoCmd_Buffer(phost,DISPLAY());
    Gpu_CoCmd_Swap(phost);
    App_Flush_Co_Buffer(phost);
    Gpu_Hal_WaitCmdfifo_empty(phost);

    do
    {
        Gpu_CoCmd_Dlstart(phost);   
        App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_A(256));
        App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_A(256));
        App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_B(0));
        App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_C(0));
        App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_D(0));
        App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_E(256));
        App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_F(0));  
        App_WrCoCmd_Buffer(phost,SAVE_CONTEXT());    
        App_WrCoCmd_Buffer(phost,COLOR_RGB(219,180,150));
        App_WrCoCmd_Buffer(phost,COLOR_A(220));
        App_WrCoCmd_Buffer(phost,BEGIN(EDGE_STRIP_A));
        App_WrCoCmd_Buffer(phost,VERTEX2F(0,DispHeight*16));
        App_WrCoCmd_Buffer(phost,VERTEX2F(DispWidth*16,DispHeight*16));
        App_WrCoCmd_Buffer(phost,COLOR_A(255));
        App_WrCoCmd_Buffer(phost,RESTORE_CONTEXT());    
        App_WrCoCmd_Buffer(phost,COLOR_RGB(0,0,0));
        // INFORMATION 
        Gpu_CoCmd_Text(phost,DispWidth/2,20,28,OPT_CENTERX|OPT_CENTERY,info[0]);
        Gpu_CoCmd_Text(phost,DispWidth/2,60,26,OPT_CENTERX|OPT_CENTERY,info[1]);
        Gpu_CoCmd_Text(phost,DispWidth/2,90,26,OPT_CENTERX|OPT_CENTERY,info[2]);  
        Gpu_CoCmd_Text(phost,DispWidth/2,120,26,OPT_CENTERX|OPT_CENTERY,info[3]);  
        Gpu_CoCmd_Text(phost,DispWidth/2,DispHeight-30,26,OPT_CENTERX|OPT_CENTERY,"Click to play");
        if(sk!='P')
        App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
        else
        App_WrCoCmd_Buffer(phost,COLOR_RGB(100,100,100));
        App_WrCoCmd_Buffer(phost,BEGIN(FTPOINTS));   
        App_WrCoCmd_Buffer(phost,POINT_SIZE(20*16));
        App_WrCoCmd_Buffer(phost,TAG('P'));
        App_WrCoCmd_Buffer(phost,VERTEX2F((DispWidth/2)*16,(DispHeight-60)*16));
        App_WrCoCmd_Buffer(phost,COLOR_RGB(180,35,35));

        App_WrCoCmd_Buffer(phost,BEGIN(BITMAPS));
        App_WrCoCmd_Buffer(phost,VERTEX2II((DispWidth/2)-14,(DispHeight-75),START_ICON_HANDLE,4));
        App_WrCoCmd_Buffer(phost,END());

        //
        //    App_WrCoCmd_Buffer(phost, BITMAP_HANDLE(LOGO_ICON_HANDLE));
        //    App_WrCoCmd_Buffer(phost, BITMAP_SOURCE(RAM_G));      // Starting address in gram
        //    App_WrCoCmd_Buffer(phost,BITMAP_LAYOUT(L4, 20, 40));  // format
        //    App_WrCoCmd_Buffer(phost,BITMAP_SIZE(NEAREST, BORDER, BORDER, 40, 40  ));
        //
        //    App_WrCoCmd_Buffer(phost,BEGIN(BITMAPS));
        //    App_WrCoCmd_Buffer(phost,VERTEX2II(10,60,LOGO_ICON_HANDLE,0));
        //    App_WrCoCmd_Buffer(phost,END());

        App_WrCoCmd_Buffer(phost,DISPLAY());
        Gpu_CoCmd_Swap(phost);
        App_Flush_Co_Buffer(phost);
        Gpu_Hal_WaitCmdfifo_empty(phost);
    }while(App_Read_Tag(phost)!='P');
    App_Play_Sound(phost, 0x50,255,0xc0);
}


/*************************************************************************************************
                        THIS PART DRAW BRIDGETEK LOGO
*************************************************************************************************/
//#define LOGO_SCALE_ANIMATION
/* Use bridgetek animation as default */
#ifndef LOGO_SCALE_ANIMATION
PROGMEM prog_uchar8_t logo_graphics[6224] = {
120, 218, 237, 93, 127, 112, 36, 71, 117, 158, 221, 213, 207, 149, 116,
44, 190, 56, 56, 72, 156, 20, 2, 4, 167, 184, 72, 49, 113, 165, 42, 196,
58, 249, 42, 135, 77, 37, 172, 20, 199, 103, 42, 14, 142, 54, 63, 140,
147, 2, 35, 37, 113, 217, 247, 7, 70, 138, 139, 248, 206, 84, 138, 85,
202, 137, 203, 103, 130, 87, 132, 10, 96, 217, 176, 71, 8, 33, 6, 131,
46, 84, 81, 16, 27, 103, 239, 236, 194, 21, 59, 94, 246, 226, 51, 196,
196, 216, 123, 62, 73, 43, 233, 36, 109, 103, 102, 119, 167, 231, 189,
158, 215, 61, 61, 163, 93, 157, 118, 153, 249, 235, 116, 253, 230, 125,
243, 125, 211, 63, 94, 191, 238, 233, 53, 140, 250, 92, 209, 175, 104,
24, 237, 95, 209, 112, 84, 208, 48, 218, 207, 86, 52, 28, 105, 24, 237,
103, 222, 70, 166, 35, 111, 35, 211, 145, 167, 145, 229, 200, 211, 200,
114, 228, 101, 84, 113, 228, 101, 84, 113, 228, 97, 84, 117, 228, 97, 84,
117, 164, 54, 170, 57, 82, 27, 213, 28, 41, 141, 108, 71, 74, 35, 219,
145, 202, 136, 59, 82, 25, 113, 71, 10, 35, 199, 145, 194, 200, 113, 36,
55, 2, 142, 228, 70, 192, 145, 212, 8, 58, 146, 26, 65, 71, 50, 35, 228,
72, 102, 132, 28, 73, 140, 176, 35, 137, 17, 118, 68, 27, 9, 142, 104,
35, 193, 17, 105, 36, 58, 34, 141, 68, 71, 148, 145, 203, 17, 101, 228,
114, 68, 24, 185, 29, 17, 70, 110, 71, 110, 35, 194, 145, 219, 136, 112,
228, 50, 162, 28, 185, 140, 40, 71, 162, 17, 233, 72, 52, 34, 29, 9, 70,
180, 35, 193, 136, 118, 132, 141, 36, 142, 176, 145, 196, 17, 50, 146,
57, 66, 70, 50, 71, 208, 72, 234, 8, 26, 73, 29, 1, 35, 185, 35, 96, 36,
119, 228, 24, 41, 28, 57, 70, 10, 71, 220, 72, 229, 136, 27, 169, 28,
217, 70, 74, 71, 182, 145, 210, 81, 205, 72, 237, 168, 102, 164, 118,
84, 53, 242, 112, 84, 53, 242, 112, 84, 49, 242, 114, 84, 49, 242, 114,
100, 25, 121, 58, 178, 140, 60, 29, 153, 70, 222, 142, 76, 35, 111, 71,
166, 81, 65, 195, 168, 141, 105, 24, 69, 138, 58, 112, 73, 29, 163, 182,
162, 134, 145, 134, 43, 211, 200, 219, 149, 245, 238, 146, 58, 70, 158,
174, 42, 245, 41, 169, 99, 228, 229, 170, 90, 199, 147, 58, 70, 30, 174,
106, 237, 46, 169, 99, 164, 118, 101, 247, 5, 73, 29, 35, 165, 43, 222,
63, 37, 117, 140, 84, 174, 156, 62, 51, 169, 99, 164, 112, 5, 250, 241,
164, 142, 145, 220, 21, 28, 91, 146, 58, 70, 82, 87, 104, 188, 75, 234,
24, 201, 92, 225, 49, 56, 169, 99, 36, 113, 37, 196, 5, 73, 29, 35, 218,
149, 24, 171, 36, 117, 140, 72, 87, 174, 248, 41, 169, 99, 68, 185, 114,
199, 116, 73, 29, 35, 194, 21, 17, 103, 38, 117, 140, 220, 174, 168,
216, 55, 169, 99, 228, 114, 69, 198, 227, 163, 58, 70, 177, 162, 134,
145, 232, 138, 54, 18, 92, 73, 230, 45, 163, 58, 70, 216, 149, 108, 46,
53, 170, 99, 132, 92, 73, 231, 119, 163, 58, 70, 208, 149, 124, 206, 57,
170, 99, 4, 92, 41, 230, 193, 163, 58, 70, 142, 43, 213, 220, 124, 84,
199, 136, 187, 82, 230, 11, 70, 117, 140, 108, 87, 234, 28, 198, 168,
142, 81, 205, 149, 71, 94, 101, 84, 199, 168, 234, 202, 43, 215, 51,
170, 99, 84, 113, 229, 153, 127, 26, 213, 49, 178, 92, 121, 231, 196,
70, 117, 140, 76, 87, 26, 121, 186, 81, 29, 163, 88, 81, 195, 200, 24,
213, 49, 138, 125, 95, 39, 87, 25, 49, 118, 242, 122, 125, 128, 123,
218, 254, 70, 39, 11, 42, 94, 151, 107, 189, 94, 241, 250, 57, 157, 74,
232, 186, 58, 138, 129, 110, 203, 176, 32, 183, 245, 179, 32, 183, 69,
114, 129, 110, 27, 96, 65, 110, 179, 193, 124, 222, 54, 192, 130, 220,
198, 193, 252, 221, 54, 192, 130, 220, 230, 128, 249, 186, 109, 128,
5, 185, 13, 128, 249, 185, 109, 128, 5, 185, 13, 130, 177, 23, 3, 129,
253, 40, 16, 88, 121, 36, 16, 216, 217, 16, 44, 4, 11, 193, 66, 176, 16,
172, 105, 192, 10, 129, 192, 222, 20, 130, 133, 96, 33, 88, 8, 22, 130,
133, 96, 33, 88, 8, 246, 211, 4, 246, 66, 48, 176, 161, 221, 9, 54, 16,
8, 204, 200, 6, 2, 235, 10, 6, 54, 30, 8, 204, 200, 5, 2, 107, 15, 6,
214, 23, 8, 12, 80, 243, 3, 102, 164, 3, 129, 57, 138, 248, 2, 139, 4,
3, 139, 5, 2, 115, 244, 95, 242, 151, 240, 181, 111, 155, 247, 117, 27,
175, 145, 211, 190, 110, 235, 180, 111, 155, 51, 2, 113, 243, 37, 164,
193, 55, 167, 108, 38, 252, 220, 22, 229, 239, 237, 223, 125, 193, 21,
130, 193, 101, 130, 193, 57, 45, 192, 23, 220, 62, 22, 8, 174, 147, 5,
131, 43, 4, 131, 155, 10, 6, 23, 103, 129, 224, 96, 234, 192, 15, 220,
47, 7, 131, 131, 203, 192, 126, 224, 70, 67, 184, 16, 46, 132, 11, 225,
66, 184, 16, 238, 98, 195, 157, 14, 6, 183, 21, 194, 133, 112, 33, 92,
8, 23, 194, 237, 90, 184, 161, 96, 112, 79, 133, 112, 33, 92, 8, 183,
227, 112, 79, 7, 131, 99, 95, 14, 6, 183, 18, 12, 206, 207, 30, 195,
209, 96, 183, 233, 237, 137, 87, 193, 249, 186, 77, 111, 223, 188, 2,
206, 223, 109, 122, 123, 235, 229, 112, 62, 111, 211, 219, 127, 239,
190, 174, 10, 118, 91, 100, 49, 208, 109, 70, 119, 176, 219, 140, 223,
14, 118, 91, 229, 190, 0, 183, 25, 191, 184, 24, 232, 54, 195, 248, 133,
131, 164, 92, 232, 175, 71, 18, 40, 7, 10, 103, 130, 89, 56, 193, 139,
163, 105, 98, 22, 206, 254, 226, 104, 14, 153, 133, 83, 195, 56, 154,
96, 102, 225, 188, 49, 142, 102, 159, 89, 56, 169, 236, 70, 83, 83, 97,
198, 153, 68, 133, 109, 69, 217, 199, 155, 41, 248, 169, 82, 201, 130,
79, 130, 34, 167, 176, 84, 125, 236, 36, 40, 178, 11, 75, 54, 221, 36,
40, 170, 22, 150, 28, 153, 146, 160, 200, 42, 44, 65, 121, 147, 41, 185,
244, 170, 235, 29, 99, 224, 143, 104, 1, 238, 68, 222, 207, 202, 99, 160,
8, 110, 83, 182, 62, 205, 229, 133, 213, 143, 121, 95, 4, 69, 188, 208,
254, 206, 247, 69, 80, 84, 43, 116, 62, 1, 126, 209, 128, 31, 13, 155,
133, 240, 235, 224, 179, 232, 51, 224, 23, 162, 104, 33, 52, 142, 150,
28, 65, 134, 220, 90, 149, 26, 64, 43, 91, 188, 176, 186, 96, 53, 128,
134, 193, 90, 161, 61, 184, 13, 160, 17, 178, 82, 232, 140, 123, 3, 104,
240, 52, 11, 225, 144, 56, 128, 198, 213, 200, 147, 104, 180, 188, 68,
245, 30, 226, 232, 253, 25, 89, 244, 114, 227, 232, 205, 91, 117, 184,
132, 10, 97, 113, 22, 85, 168, 56, 170, 109, 118, 237, 47, 161, 66, 187,
56, 139, 42, 113, 28, 213, 112, 184, 38, 92, 194, 127, 90, 197, 240, 69,
136, 197, 67, 168, 184, 132, 144, 170, 200, 113, 84, 232, 44, 54, 165,
96, 247, 233, 112, 202, 32, 74, 213, 98, 160, 71, 6, 201, 97, 21, 35,
45, 51, 72, 74, 163, 187, 132, 223, 195, 152, 178, 181, 224, 73, 78,
39, 222, 183, 63, 83, 134, 55, 119, 226, 109, 253, 51, 12, 182, 166,
78, 188, 235, 127, 134, 193, 246, 212, 137, 63, 10, 152, 97, 176, 69,
117, 226, 111, 6, 102, 24, 108, 83, 157, 248, 147, 130, 25, 180, 218,
13, 10, 217, 58, 42, 180, 22, 58, 115, 168, 16, 238, 28, 152, 131, 173,
167, 82, 8, 138, 231, 96, 251, 169, 21, 242, 226, 57, 216, 130, 120, 97,
173, 120, 14, 182, 33, 80, 88, 41, 134, 43, 175, 145, 220, 58, 146, 121,
96, 206, 208, 239, 234, 186, 240, 27, 203, 156, 198, 227, 50, 154, 181,
102, 208, 108, 180, 27, 79, 106, 51, 104, 178, 218, 141, 131, 225, 44,
138, 169, 227, 56, 86, 206, 162, 144, 59, 142, 67, 233, 44, 138, 200,
163, 66, 164, 45, 4, 236, 40, 16, 199, 33, 104, 9, 135, 150, 41, 20,
195, 149, 112, 108, 150, 66, 49, 87, 9, 199, 82, 41, 20, 146, 149, 112,
168, 149, 66, 17, 91, 9, 71, 98, 168, 89, 141, 226, 102, 21, 75, 41, 227,
42, 188, 87, 103, 0, 53, 171, 72, 14, 53, 171, 1, 212, 172, 172, 170, 3,
138, 7, 80, 179, 170, 86, 59, 94, 60, 128, 154, 149, 93, 101, 107, 197,
3, 168, 89, 57, 213, 189, 82, 60, 128, 154, 21, 218, 195, 45, 238, 205,
22, 54, 120, 195, 63, 207, 34, 79, 149, 253, 73, 78, 241, 89, 244, 20,
181, 205, 75, 118, 241, 89, 196, 128, 239, 108, 170, 22, 159, 69, 236,
193, 182, 39, 171, 248, 44, 82, 14, 237, 137, 138, 228, 240, 198, 170,
55, 170, 54, 76, 69, 113, 43, 154, 69, 141, 104, 13, 183, 161, 89, 212,
132, 214, 112, 11, 154, 69, 13, 104, 13, 245, 232, 181, 66, 59, 40, 91,
67, 99, 65, 165, 208, 137, 215, 214, 96, 145, 85, 8, 118, 246, 88, 133,
105, 6, 11, 225, 214, 164, 85, 84, 184, 138, 246, 45, 89, 251, 68, 210,
168, 213, 116, 225, 33, 50, 141, 218, 76, 23, 30, 92, 211, 168, 197, 116,
225, 97, 57, 141, 218, 75, 23, 30, 208, 211, 168, 181, 116, 41, 26, 75,
59, 26, 159, 38, 97, 75, 105, 135, 45, 193, 152, 132, 237, 164, 29, 182,
3, 171, 200, 41, 236, 128, 173, 192, 222, 210, 176, 12, 138, 204, 107,
12, 20, 85, 11, 59, 28, 205, 198, 208, 70, 136, 101, 88, 196, 46, 192,
34, 54, 223, 142, 138, 106, 179, 41, 123, 95, 79, 63, 40, 50, 64, 225,
188, 5, 222, 15, 138, 156, 194, 249, 234, 67, 247, 131, 34, 187, 112,
222, 38, 219, 15, 138, 170, 133, 243, 142, 72, 253, 160, 200, 42, 156,
135, 226, 254, 172, 17, 94, 187, 241, 234, 165, 14, 218, 40, 191, 242,
252, 227, 119, 31, 244, 233, 105, 15, 112, 112, 74, 102, 212, 134, 59,
150, 6, 243, 168, 94, 27, 127, 209, 26, 60, 24, 251, 78, 162, 53, 120,
176, 31, 182, 8, 15, 246, 233, 22, 225, 225, 35, 19, 189, 171, 121, 176,
147, 45, 194, 99, 173, 69, 120, 176, 137, 22, 225, 113, 170, 69, 120,
44, 181, 8, 143, 245, 22, 225, 81, 110, 17, 30, 218, 13, 125, 183, 243,
152, 107, 17, 30, 39, 90, 132, 199, 169, 22, 225, 113, 174, 69, 120,
232, 14, 32, 125, 175, 58, 215, 201, 139, 204, 35, 159, 207, 23, 116,
78, 73, 12, 124, 237, 12, 143, 202, 88, 241, 150, 27, 49, 143, 213, 38,
229, 33, 158, 235, 216, 188, 60, 218, 2, 70, 238, 187, 141, 7, 62, 254,
121, 189, 121, 121, 164, 91, 132, 199, 1, 134, 114, 188, 77, 203, 99,
184, 69, 120, 12, 162, 180, 98, 243, 242, 120, 93, 43, 242, 104, 226,
122, 181, 167, 21, 121, 172, 183, 8, 143, 181, 86, 231, 113, 233, 61,
15, 255, 224, 249, 71, 143, 122, 30, 241, 177, 247, 125, 31, 127, 242,
249, 135, 110, 79, 168, 121, 188, 237, 147, 79, 190, 242, 196, 67, 127,
105, 104, 120, 123, 236, 123, 249, 87, 159, 127, 226, 161, 251, 255,
216, 23, 15, 30, 39, 222, 184, 192, 175, 207, 24, 70, 196, 14, 139,
203, 127, 94, 45, 238, 118, 138, 23, 224, 156, 222, 182, 219, 248, 67,
21, 143, 143, 212, 254, 243, 255, 126, 199, 250, 235, 1, 199, 213, 16,
102, 251, 69, 248, 104, 155, 159, 187, 58, 0, 143, 52, 154, 147, 92,
229, 228, 228, 71, 84, 243, 193, 247, 58, 255, 251, 183, 114, 30, 55,
57, 207, 48, 141, 3, 60, 200, 35, 250, 41, 87, 86, 234, 239, 180, 121,
148, 72, 30, 112, 135, 231, 89, 5, 143, 55, 194, 69, 136, 49, 25, 15,
184, 35, 110, 115, 68, 198, 35, 154, 33, 166, 171, 255, 53, 164, 201,
99, 133, 228, 49, 9, 111, 75, 200, 121, 44, 226, 166, 38, 225, 177, 40,
116, 144, 36, 143, 72, 134, 156, 119, 255, 175, 230, 56, 184, 76, 241,
136, 34, 87, 179, 82, 30, 61, 24, 115, 158, 230, 33, 90, 209, 60, 100,
191, 56, 243, 45, 189, 248, 234, 60, 197, 35, 142, 60, 157, 145, 242,
152, 18, 146, 197, 52, 143, 25, 49, 165, 76, 241, 144, 254, 186, 78,
57, 165, 21, 239, 158, 161, 120, 28, 112, 167, 84, 40, 30, 174, 159,
209, 249, 19, 138, 71, 84, 180, 154, 165, 120, 168, 127, 145, 202, 123,
254, 113, 146, 226, 177, 232, 158, 194, 83, 60, 186, 68, 200, 11, 20,
143, 30, 209, 106, 149, 224, 17, 41, 200, 83, 233, 35, 18, 30, 83, 100,
126, 23, 240, 216, 16, 158, 78, 198, 163, 79, 107, 201, 110, 88, 101,
53, 36, 147, 4, 31, 248, 68, 242, 128, 167, 92, 87, 247, 17, 185, 102,
187, 204, 21, 218, 83, 60, 134, 181, 120, 164, 117, 120, 236, 83, 216,
92, 160, 121, 196, 232, 245, 15, 57, 220, 150, 140, 199, 148, 22, 143,
130, 14, 143, 140, 218, 136, 226, 209, 79, 135, 87, 105, 245, 90, 15,
197, 35, 171, 195, 35, 198, 52, 120, 68, 61, 214, 54, 8, 30, 111, 43,
208, 233, 221, 0, 60, 22, 117, 120, 116, 232, 240, 232, 84, 26, 189, 38,
228, 119, 143, 31, 63, 254, 176, 240, 150, 103, 183, 195, 35, 167, 195,
35, 174, 195, 163, 71, 105, 180, 226, 153, 111, 7, 155, 166, 3, 240, 40,
234, 240, 232, 211, 225, 177, 199, 99, 49, 214, 139, 199, 178, 177, 29,
30, 90, 91, 37, 6, 117, 120, 140, 195, 255, 123, 233, 250, 43, 14, 221,
137, 2, 120, 47, 30, 112, 83, 104, 163, 120, 12, 235, 240, 72, 187, 102,
218, 232, 113, 188, 120, 192, 15, 107, 26, 197, 3, 69, 15, 207, 28, 188,
228, 154, 34, 193, 3, 246, 24, 167, 221, 17, 238, 144, 154, 199, 122, 66,
194, 163, 124, 239, 155, 175, 201, 249, 228, 241, 210, 193, 189, 191,
239, 197, 99, 221, 213, 166, 135, 220, 99, 204, 188, 123, 128, 31, 81,
242, 120, 9, 77, 42, 211, 66, 179, 137, 251, 227, 81, 233, 49, 166, 8,
30, 227, 162, 212, 81, 130, 71, 209, 213, 133, 162, 238, 122, 76, 193,
163, 252, 57, 188, 61, 49, 45, 72, 210, 230, 143, 199, 146, 40, 181, 205,
99, 210, 37, 117, 193, 205, 3, 122, 170, 30, 88, 216, 142, 54, 43, 200,
121, 124, 71, 76, 132, 164, 5, 87, 17, 127, 60, 206, 136, 195, 25, 197,
99, 214, 213, 24, 170, 60, 34, 30, 60, 82, 114, 30, 47, 223, 119, 181,
156, 199, 132, 127, 30, 39, 69, 112, 138, 199, 180, 43, 156, 25, 114,
135, 37, 254, 120, 152, 215, 183, 235, 206, 163, 45, 32, 143, 216, 182,
120, 224, 134, 126, 49, 121, 180, 109, 143, 7, 250, 24, 239, 98, 242,
104, 223, 38, 15, 16, 38, 54, 55, 143, 141, 22, 225, 33, 137, 219, 155,
143, 199, 114, 139, 240, 40, 39, 90, 131, 135, 83, 177, 154, 135, 71,
249, 176, 121, 253, 233, 35, 152, 199, 255, 236, 58, 30, 196, 149, 162,
242, 37, 239, 101, 94, 235, 6, 213, 77, 164, 119, 216, 215, 237, 187,
147, 7, 78, 115, 108, 74, 121, 120, 239, 235, 187, 200, 60, 198, 201, 76,
88, 243, 241, 192, 25, 140, 137, 166, 229, 209, 201, 168, 68, 117, 243,
241, 192, 119, 157, 106, 90, 30, 56, 227, 122, 174, 105, 121, 68, 133,
228, 105, 179, 242, 64, 179, 97, 190, 64, 216, 124, 60, 112, 222, 105,
169, 69, 120, 44, 183, 8, 143, 149, 22, 225, 81, 106, 17, 30, 107, 45,
194, 99, 125, 183, 241, 184, 235, 205, 174, 43, 161, 193, 227, 194, 110,
227, 49, 173, 187, 31, 46, 228, 177, 155, 121, 172, 135, 60, 46, 50, 143,
200, 238, 238, 119, 181, 121, 196, 90, 132, 7, 254, 176, 104, 181, 105,
121, 180, 55, 5, 143, 215, 131, 139, 230, 209, 181, 187, 227, 171, 105,
119, 118, 42, 229, 189, 239, 117, 87, 196, 237, 109, 129, 120, 76, 238,
186, 249, 96, 44, 8, 143, 104, 174, 17, 121, 134, 83, 117, 94, 175, 245,
230, 241, 81, 214, 136, 188, 207, 57, 177, 229, 5, 94, 63, 159, 117, 111,
238, 74, 185, 246, 145, 29, 127, 164, 32, 196, 246, 39, 130, 243, 40,
138, 221, 197, 155, 60, 120, 104, 237, 103, 152, 115, 59, 159, 240, 94,
255, 216, 78, 62, 177, 40, 228, 87, 225, 105, 115, 36, 15, 173, 253, 37,
39, 220, 175, 104, 76, 131, 199, 116, 112, 30, 232, 213, 110, 30, 189,
37, 75, 197, 9, 227, 98, 44, 231, 181, 223, 103, 201, 189, 29, 80, 135,
199, 72, 157, 120, 136, 151, 175, 253, 87, 80, 129, 45, 107, 115, 91,
100, 6, 61, 163, 55, 143, 242, 54, 214, 63, 114, 190, 121, 184, 174, 33,
98, 39, 240, 214, 67, 71, 239, 193, 91, 106, 135, 188, 121, 92, 216, 6,
143, 69, 29, 30, 131, 58, 60, 212, 155, 255, 88, 194, 155, 199, 202, 54,
120, 164, 117, 120, 212, 97, 191, 232, 166, 198, 57, 19, 175, 109, 131,
135, 214, 190, 112, 173, 253, 187, 106, 163, 117, 63, 231, 176, 4, 224,
49, 174, 195, 163, 83, 135, 71, 135, 71, 157, 233, 213, 238, 174, 130,
240, 24, 212, 225, 17, 213, 225, 17, 41, 170, 235, 140, 143, 115, 163,
2, 240, 232, 213, 225, 161, 238, 213, 134, 116, 170, 232, 73, 111, 30,
47, 108, 135, 135, 187, 202, 20, 9, 30, 51, 58, 60, 148, 189, 193, 152,
55, 143, 233, 237, 240, 112, 85, 153, 242, 56, 193, 99, 159, 14, 15, 85,
134, 119, 195, 251, 92, 156, 11, 198, 118, 120, 184, 170, 204, 202, 48,
193, 67, 124, 196, 173, 28, 193, 67, 53, 22, 45, 121, 243, 152, 219, 30,
15, 177, 195, 154, 167, 120, 136, 143, 248, 84, 150, 226, 209, 163, 172,
51, 30, 60, 214, 140, 237, 241, 232, 22, 170, 85, 130, 228, 49, 128, 141,
134, 72, 30, 242, 143, 123, 74, 94, 61, 10, 219, 154, 216, 38, 15, 225,
107, 185, 21, 131, 228, 17, 65, 213, 239, 172, 65, 243, 232, 150, 46,
126, 120, 241, 216, 74, 25, 219, 228, 97, 188, 3, 41, 61, 65, 243, 64,
95, 19, 175, 39, 36, 60, 170, 191, 4, 228, 190, 190, 236, 213, 195, 191,
60, 97, 108, 155, 7, 154, 57, 61, 109, 72, 120, 128, 71, 180, 130, 114,
9, 15, 227, 253, 82, 26, 114, 30, 229, 103, 255, 200, 48, 182, 207, 195,
104, 115, 136, 108, 36, 164, 60, 140, 27, 107, 3, 203, 143, 199, 12, 57,
15, 227, 90, 215, 144, 249, 114, 237, 4, 129, 182, 195, 212, 117, 253,
59, 127, 222, 253, 160, 111, 5, 6, 196, 65, 201, 208, 19, 248, 138, 199,
136, 253, 67, 13, 242, 171, 214, 35, 93, 234, 24, 93, 135, 110, 143, 221,
243, 216, 15, 126, 242, 232, 199, 36, 249, 18, 254, 122, 127, 239, 27,
176, 222, 63, 126, 155, 177, 131, 87, 236, 134, 35, 199, 143, 222, 124,
181, 254, 13, 114, 30, 214, 216, 122, 232, 150, 35, 247, 28, 191, 255,
232, 237, 55, 31, 52, 118, 249, 165, 228, 209, 68, 87, 200, 35, 228, 17,
242, 104, 57, 30, 61, 226, 83, 195, 24, 32, 209, 60, 60, 122, 69, 30,
48, 184, 52, 154, 146, 199, 152, 152, 92, 47, 55, 39, 143, 57, 113, 86,
191, 213, 156, 60, 214, 204, 230, 16, 153, 66, 153, 194, 166, 108, 231,
108, 227, 243, 247, 124, 83, 72, 33, 52, 205, 213, 173, 57, 177, 222,
237, 151, 234, 136, 149, 165, 38, 226, 161, 202, 221, 158, 105, 34, 30,
170, 4, 239, 124, 51, 197, 33, 138, 229, 183, 137, 102, 226, 145, 145,
103, 108, 154, 42, 46, 148, 175, 16, 46, 55, 21, 143, 120, 107, 52, 15,
121, 67, 47, 55, 217, 236, 99, 82, 194, 227, 197, 38, 155, 56, 73, 70,
144, 173, 38, 123, 29, 134, 241, 97, 146, 199, 151, 154, 110, 38, 27,
165, 86, 105, 158, 105, 194, 41, 121, 135, 123, 44, 252, 118, 83, 230,
22, 98, 15, 210, 9, 232, 230, 187, 174, 249, 130, 211, 192, 191, 118,
155, 209, 196, 215, 222, 67, 55, 28, 57, 122, 251, 7, 14, 255, 138, 17,
94, 254, 175, 134, 103, 153, 66, 128, 16, 32, 4, 8, 1, 234, 3, 16, 165,
119, 211, 132, 0, 33, 64, 8, 16, 2, 132, 0, 33, 64, 8, 176, 219, 0, 46,
121, 247, 199, 191, 254, 189, 252, 171, 249, 39, 22, 238, 191, 45, 225,
23, 160, 123, 97, 225, 97, 152, 141, 170, 254, 192, 202, 63, 1, 139, 75,
209, 15, 147, 108, 253, 219, 117, 30, 0, 240, 55, 95, 38, 100, 31, 33,
56, 63, 52, 24, 249, 136, 171, 240, 43, 9, 37, 192, 56, 218, 220, 236,
5, 16, 249, 40, 81, 90, 217, 9, 39, 5, 200, 224, 165, 104, 15, 128, 91,
233, 189, 200, 9, 5, 64, 17, 111, 133, 87, 3, 200, 18, 210, 63, 148, 3,
180, 11, 217, 119, 53, 64, 214, 59, 225, 45, 2, 196, 133, 253, 136, 74,
128, 184, 198, 134, 109, 17, 96, 80, 216, 14, 173, 4, 200, 104, 236,
108, 23, 1, 166, 132, 175, 42, 84, 0, 49, 157, 47, 125, 68, 128, 69,
97, 33, 71, 5, 160, 58, 160, 158, 159, 253, 40, 0, 68, 197, 5, 97, 21,
192, 184, 206, 238, 127, 1, 160, 83, 92, 169, 85, 1, 40, 63, 183, 56,
69, 3, 244, 137, 38, 10, 0, 245, 119, 41, 37, 26, 0, 46, 73, 205, 122,
0, 116, 104, 125, 89, 32, 0, 164, 197, 79, 115, 20, 0, 234, 143, 152,
182, 104, 128, 130, 104, 161, 0, 192, 63, 96, 119, 253, 21, 87, 126,
144, 250, 198, 2, 3, 184, 63, 177, 86, 0, 64, 57, 171, 63, 137, 112,
128, 88, 117, 197, 0, 221, 174, 229, 126, 5, 192, 148, 203, 184, 147,
168, 167, 24, 192, 189, 51, 186, 253, 240, 225, 247, 193, 205, 43, 96,
35, 113, 218, 181, 104, 143, 154, 246, 44, 5, 48, 73, 245, 38, 178, 49,
57, 227, 122, 154, 8, 241, 97, 7, 6, 32, 55, 212, 201, 0, 178, 232, 3,
165, 202, 69, 244, 216, 8, 32, 66, 238, 228, 145, 1, 44, 170, 1, 78,
16, 0, 29, 228, 184, 46, 3, 200, 249, 7, 128, 251, 232, 207, 123, 2,
20, 252, 3, 12, 187, 111, 81, 0, 20, 253, 3, 204, 144, 31, 91, 201,
0, 152, 127, 128, 2, 185, 43, 179, 126, 0, 123, 233, 237, 96, 245, 3,
184, 140, 172, 68, 117, 4, 216, 71, 86, 162, 58, 2, 76, 82, 223, 155,
214, 19, 32, 39, 124, 238, 83, 119, 0, 70, 207, 21, 116, 0, 136, 75,
13, 176, 97, 52, 24, 160, 212, 104, 128, 215, 26, 13, 112, 162, 209, 0,
179, 141, 6, 56, 213, 104, 128, 165, 70, 3, 172, 53, 26, 96, 171, 33,
0, 69, 38, 110, 175, 174, 51, 192, 12, 253, 185, 106, 253, 0, 134, 233,
253, 149, 245, 3, 232, 167, 119, 168, 213, 15, 224, 13, 180, 167, 250,
1, 92, 66, 205, 18, 245, 0, 190, 69, 156, 58, 73, 68, 21, 57, 114, 83,
165, 143, 1, 199, 43, 227, 53, 67, 102, 27, 234, 8, 112, 128, 236, 176,
235, 8, 208, 71, 14, 57, 117, 4, 232, 34, 7, 205, 58, 2, 180, 145, 195,
126, 29, 1, 80, 192, 156, 106, 4, 64, 134, 186, 167, 158, 0, 147, 84,
240, 232, 3, 32, 2, 142, 181, 163, 26, 26, 58, 30, 97, 53, 0, 192, 148,
199, 252, 0, 255, 122, 115, 128, 25, 142, 39, 0, 58, 24, 97, 196, 255,
28, 205, 19, 128, 58, 70, 12, 1, 108, 168, 103, 153, 158, 0, 232, 79,
234, 116, 139, 77, 73, 253, 168, 217, 206, 120, 2, 76, 17, 161, 75, 84,
18, 117, 103, 221, 3, 96, 198, 99, 166, 143, 79, 46, 90, 167, 0, 102,
233, 92, 197, 107, 110, 213, 200, 92, 5, 250, 164, 157, 60, 217, 30, 4,
76, 105, 215, 127, 183, 123, 102, 91, 112, 198, 103, 140, 0, 96, 207,
221, 125, 228, 129, 175, 87, 254, 31, 237, 12, 182, 126, 73, 222, 248,
3, 111, 128, 24, 145, 144, 137, 72, 194, 178, 65, 148, 136, 125, 232,
232, 145, 79, 49, 207, 132, 20, 174, 220, 103, 100, 163, 123, 21, 64,
125, 140, 19, 157, 82, 195, 194, 174, 16, 77, 22, 0, 116, 171, 1, 134,
104, 128, 3, 68, 179, 45, 208, 0, 237, 65, 210, 154, 56, 9, 152, 144,
4, 200, 91, 146, 119, 163, 145, 152, 197, 188, 83, 238, 246, 9, 159,
46, 27, 32, 181, 140, 15, 203, 172, 181, 246, 3, 18, 0, 229, 121, 98,
231, 53, 22, 137, 236, 246, 185, 71, 2, 160, 250, 200, 138, 135, 231,
46, 128, 172, 155, 102, 151, 4, 64, 117, 226, 147, 108, 129, 2, 183,
207, 77, 137, 35, 187, 134, 40, 142, 98, 90, 147, 174, 163, 13, 18, 117,
57, 45, 1, 80, 36, 248, 191, 36, 5, 232, 33, 154, 123, 143, 108, 6, 39,
165, 176, 33, 95, 9, 236, 160, 130, 133, 156, 4, 160, 221, 255, 66, 29,
238, 59, 237, 186, 214, 47, 155, 131, 94, 78, 251, 255, 190, 106, 61,
57, 71, 189, 170, 15, 203, 38, 185, 191, 68, 249, 127, 70, 185, 96, 61,
67, 121, 138, 220, 41, 155, 69, 255, 154, 171, 35, 217, 252, 152, 122,
69, 124, 152, 10, 93, 12, 227, 61, 255, 73, 3, 24, 198, 181, 143, 129,
106, 92, 126, 226, 239, 241, 194, 250, 149, 224, 144, 147, 234, 255,
160, 3, 84, 134, 128, 233, 91, 111, 57, 114, 252, 216, 237, 55, 255,
230, 21, 174, 163, 84, 34, 87, 86, 78, 254, 56, 118, 199, 7, 14, 54,
209, 215, 233, 63, 53, 215, 222, 235, 135, 52, 172, 126, 230, 250, 29,
124, 119, 111, 249, 93, 77, 176, 200, 149, 7, 181, 74, 162, 135, 92,
123, 254, 173, 30, 232, 39, 247, 218, 127, 153, 61, 97, 249, 113, 158,
94, 234, 227, 221, 203, 140, 117, 194, 145, 211, 142, 46, 47, 130, 51,
180, 114, 224, 48, 196, 105, 51, 218, 227, 191, 164, 247, 46, 179, 159,
28, 169, 167, 30, 157, 104, 248, 155, 66, 173, 248, 253, 213, 175, 220,
107, 113, 112, 202, 156, 29, 217, 41, 160, 113, 43, 110, 158, 228, 107,
40, 209, 44, 248, 150, 166, 23, 46, 28, 68, 30, 132, 253, 150, 117, 18,
211, 63, 243, 200, 125, 194, 226, 89, 235, 219, 103, 249, 205, 214, 72,
96, 251, 221, 131, 79, 202, 230, 153, 131, 104, 1, 246, 162, 5, 187,
55, 234, 180, 6, 135, 46, 158, 193, 104, 19, 146, 86, 219, 191, 186,
208, 64, 50, 3, 195, 251, 10, 141, 211, 156, 219, 180, 249, 218, 237,
160, 115, 210, 50, 154, 226, 93, 230, 40, 92, 222, 234, 131, 201, 217,
253, 40, 79, 56, 9, 250, 217, 41, 19, 51, 106, 70, 7, 29, 40, 154, 237,
69, 9, 112, 65, 44, 158, 138, 234, 65, 249, 77, 40, 214, 107, 64, 172,
125, 66, 126, 172, 161, 98, 13, 195, 153, 151, 40, 150, 249, 128, 142,
88, 5, 56, 9, 68, 98, 45, 66, 37, 170, 227, 119, 77, 254, 65, 115, 62,
209, 201, 86, 77, 177, 202, 11, 57, 94, 155, 123, 217, 230, 195, 69,
126, 3, 20, 235, 185, 111, 58, 41, 142, 113, 182, 241, 103, 69, 46, 41,
20, 107, 5, 136, 53, 195, 46, 124, 144, 254, 81, 129, 160, 87, 219, 194,
2, 99, 95, 91, 224, 49, 248, 115, 11, 11, 159, 231, 15, 184, 246, 33,
30, 254, 187, 196, 154, 118, 196, 50, 43, 199, 103, 63, 193, 99, 125,
40, 150, 57, 155, 251, 215, 79, 112, 37, 204, 23, 115, 223, 162, 221,
152, 122, 76, 179, 30, 83, 140, 14, 51, 254, 234, 224, 225, 97, 175,
217, 110, 250, 121, 68, 6, 197, 154, 53, 123, 130, 37, 46, 195, 73, 83,
176, 51, 132, 88, 235, 64, 172, 69, 243, 158, 25, 180, 76, 83, 135, 11,
172, 223, 206, 192, 212, 122, 206, 20, 132, 63, 136, 75, 172, 19, 142,
88, 221, 166, 80, 29, 224, 103, 25, 29, 177, 58, 204, 118, 22, 227, 130,
247, 154, 21, 169, 203, 174, 171, 29, 102, 167, 51, 104, 146, 182, 196,
50, 97, 198, 28, 177, 98, 92, 93, 44, 214, 30, 222, 105, 101, 205, 199,
116, 58, 127, 40, 214, 22, 16, 171, 192, 172, 3, 3, 207, 236, 140, 88,
69, 147, 65, 218, 158, 247, 185, 196, 58, 231, 136, 213, 99, 138, 16,
229, 146, 64, 177, 186, 45, 105, 120, 239, 178, 199, 44, 137, 241, 73,
140, 249, 143, 113, 211, 105, 69, 172, 69, 27, 198, 18, 203, 224, 103,
240, 96, 177, 122, 121, 159, 110, 213, 153, 246, 187, 71, 8, 177, 216,
144, 35, 150, 133, 219, 249, 215, 67, 59, 35, 150, 213, 47, 189, 235, 62,
89, 51, 92, 118, 196, 234, 51, 95, 121, 132, 123, 129, 98, 197, 45, 177,
138, 118, 199, 60, 104, 149, 28, 189, 142, 51, 78, 100, 204, 123, 106,
98, 77, 0, 177, 138, 206, 65, 213, 80, 172, 30, 48, 171, 159, 70, 73,
67, 46, 214, 22, 75, 97, 177, 234, 126, 169, 196, 2, 57, 58, 44, 86, 97,
205, 37, 214, 136, 91, 172, 30, 171, 50, 0, 177, 64, 80, 97, 222, 158,
42, 108, 26, 85, 177, 114, 246, 205, 181, 154, 149, 160, 196, 138, 123,
139, 181, 198, 230, 161, 88, 35, 59, 38, 86, 4, 157, 16, 38, 138, 149,
222, 132, 98, 173, 128, 71, 115, 137, 85, 176, 219, 24, 22, 107, 144,
253, 163, 149, 44, 183, 196, 138, 240, 14, 207, 18, 43, 202, 83, 181,
88, 172, 110, 46, 86, 14, 238, 160, 128, 98, 45, 179, 51, 187, 81, 172,
36, 75, 112, 177, 246, 40, 196, 90, 5, 98, 13, 35, 177, 122, 216, 143,
172, 191, 205, 49, 224, 87, 63, 196, 151, 24, 44, 177, 248, 24, 32, 136,
213, 197, 71, 73, 169, 88, 231, 11, 75, 92, 172, 72, 67, 142, 131, 67,
98, 125, 154, 47, 158, 136, 98, 221, 251, 206, 223, 0, 98, 253, 22, 155,
0, 98, 45, 87, 135, 3, 90, 172, 156, 35, 22, 220, 207, 97, 142, 159, 86,
224, 36, 70, 240, 107, 55, 100, 249, 168, 135, 197, 234, 4, 98, 77, 92,
118, 236, 216, 93, 132, 88, 233, 85, 36, 214, 219, 143, 29, 251, 171, 6,
138, 229, 36, 159, 35, 196, 186, 130, 35, 214, 187, 217, 156, 150, 88,
37, 44, 86, 236, 216, 177, 99, 78, 205, 181, 186, 158, 14, 52, 193, 233,
133, 113, 171, 32, 86, 7, 111, 158, 102, 80, 48, 236, 204, 126, 160, 88,
227, 27, 80, 44, 179, 242, 163, 202, 220, 0, 177, 206, 241, 120, 91, 33,
214, 65, 118, 10, 136, 181, 4, 66, 37, 40, 86, 175, 197, 104, 209, 17,
235, 92, 23, 88, 218, 89, 172, 0, 87, 107, 214, 89, 40, 214, 214, 72,
96, 177, 246, 176, 203, 118, 161, 88, 99, 133, 243, 219, 21, 107, 170,
66, 222, 148, 224, 208, 173, 96, 186, 3, 55, 254, 200, 155, 97, 74, 34,
86, 15, 187, 22, 54, 195, 201, 198, 138, 245, 236, 241, 227, 179, 116,
51, 252, 234, 29, 119, 66, 177, 50, 165, 32, 205, 176, 3, 136, 53, 88,
233, 214, 43, 163, 97, 209, 137, 224, 87, 223, 227, 44, 40, 171, 58,
248, 110, 254, 48, 80, 172, 14, 118, 12, 117, 240, 189, 13, 21, 75,
119, 52, 28, 155, 188, 32, 138, 53, 162, 213, 193, 23, 153, 147, 173,
90, 174, 137, 101, 78, 251, 102, 249, 104, 24, 115, 22, 121, 229, 161,
195, 180, 68, 172, 40, 251, 46, 10, 29, 250, 208, 152, 114, 209, 196,
26, 44, 7, 11, 29, 72, 177, 102, 236, 180, 135, 21, 58, 20, 216, 144,
225, 29, 148, 198, 73, 177, 140, 194, 250, 197, 18, 75, 17, 193, 143,
245, 176, 76, 160, 160, 148, 121, 138, 149, 225, 241, 185, 124, 186,
51, 43, 19, 43, 83, 70, 211, 157, 61, 59, 36, 150, 122, 186, 51, 214,
201, 138, 254, 167, 59, 75, 18, 177, 210, 160, 25, 154, 211, 235, 147,
26, 19, 233, 30, 158, 4, 69, 98, 77, 50, 65, 172, 115, 59, 39, 86, 242,
209, 33, 137, 88, 49, 198, 160, 88, 81, 205, 137, 116, 132, 20, 11, 101,
29, 28, 137, 176, 88, 251, 56, 124, 198, 124, 204, 94, 14, 131, 196, 26,
100, 40, 69, 243, 186, 157, 76, 209, 100, 100, 41, 154, 49, 43, 61, 170,
155, 162, 1, 33, 70, 196, 73, 241, 59, 98, 117, 50, 152, 207, 138, 243,
225, 16, 137, 21, 93, 4, 203, 23, 167, 141, 3, 188, 206, 32, 177, 122,
28, 177, 178, 102, 219, 158, 218, 169, 228, 223, 34, 155, 139, 72, 147,
127, 150, 142, 78, 242, 111, 211, 74, 143, 144, 201, 191, 173, 68, 59,
72, 254, 173, 155, 118, 23, 68, 177, 88, 158, 129, 56, 107, 217, 104,
39, 39, 210, 175, 20, 156, 220, 252, 1, 182, 245, 0, 153, 86, 62, 95,
205, 195, 243, 56, 238, 147, 245, 77, 43, 139, 98, 189, 146, 207, 255,
55, 255, 99, 227, 65, 158, 14, 112, 139, 53, 201, 197, 106, 99, 236, 63,
22, 249, 236, 183, 143, 149, 243, 249, 124, 202, 14, 108, 159, 253, 34,
15, 143, 76, 34, 223, 93, 116, 214, 104, 184, 88, 96, 17, 198, 18, 43,
194, 168, 124, 22, 92, 83, 142, 163, 191, 144, 88, 49, 71, 172, 190,
186, 47, 88, 16, 17, 124, 25, 174, 142, 172, 74, 197, 26, 116, 30, 55,
7, 119, 16, 244, 193, 153, 113, 6, 42, 81, 221, 225, 113, 138, 16, 139,
255, 2, 77, 37, 159, 197, 123, 48, 65, 44, 126, 172, 99, 101, 27, 68,
137, 202, 58, 88, 127, 217, 98, 197, 138, 96, 30, 213, 104, 177, 218,
224, 170, 148, 91, 172, 94, 71, 172, 253, 112, 101, 10, 137, 213, 143,
118, 193, 90, 75, 102, 206, 129, 156, 85, 177, 218, 242, 249, 231, 31,
229, 22, 241, 252, 83, 134, 113, 83, 190, 214, 25, 244, 228, 237, 41,
245, 173, 166, 213, 103, 157, 74, 242, 134, 2, 219, 224, 105, 208, 47,
228, 107, 46, 219, 45, 243, 59, 243, 246, 88, 106, 188, 189, 200, 126,
92, 239, 44, 205, 55, 242, 252, 41, 110, 50, 27, 16, 111, 134, 198, 175,
131, 5, 210, 59, 243, 19, 70, 87, 254, 233, 218, 95, 87, 153, 207, 215,
158, 183, 31, 178, 242, 137, 254, 103, 28, 190, 121, 222, 12, 141, 200,
173, 140, 253, 11, 71, 138, 154, 205, 250, 46, 3, 9, 19, 240, 138, 92,
161, 101, 181, 147, 103, 222, 69, 181, 193, 246, 14, 233, 149, 236, 117,
117, 33, 255, 15, 136, 52, 18, 253};

static const struct element {
  uint32_t src;
  uint16_t w, h, x0, y0;
} el[] = {
  { 0,      74, 145,  54,  48 },
  { 5365,  110, 218,  18,  11 },
  { 17355,  28,  29,   8,  66 },
  { 17761,  28,  29,   8, 145 },
  { 18167,  30,  28,  29,  45 },
  { 18587,  30,  29,  29, 167 },
  { 19022,  30,  28,  51,  23 },
  { 19442,  30,  28,  51, 189 },
  { 19862,  28,  28,  73,   1 },
  { 20254,  28,  28,  73, 211 },
  { 20646, 400, 149, 190,  47 },
  { 50446, 194, 120, 597,  47 },
  { 62086, 602,  29, 190, 183 }
};

static int32_t scaling;
static int16_t YOFFSET;

#define FX16(x)   ((int32_t)((x) * 65536.0f))

static int32_t tween(int32_t t, int32_t a, int32_t b)
{
  uint16_t tu, tu2, tu3;
  if (t < 0)
    return a;
  if (t > 65535)
    return b;
  tu = t;
  tu = ~tu;
  tu2 = ((uint32_t)tu * tu) >> 16;
  tu3 = ((uint32_t)tu2 * tu) >> 16;
  tu3 = ~tu3;
  return a + (((int32_t)tu3 * (b - a)) >> 16);
}

int32_t dc(int16_t c)
{
  return (scaling * c) >> 12;
}
static void setup_element(Gpu_Hal_Context_t *phost, const struct element *e)
{
  App_WrCoCmd_Buffer(phost, BITMAP_SOURCE(e->src));
  App_WrCoCmd_Buffer(phost, BITMAP_LAYOUT(L4, e->w / 2, e->h));
  App_WrCoCmd_Buffer(phost, BITMAP_SIZE(BILINEAR,BORDER,BORDER,(dc(e->w) >> 4) + 1,(dc(e->h) >> 4) + 1));
#ifdef FT81X_ENABLE
  App_WrCoCmd_Buffer(phost, BITMAP_LAYOUT_H((e->w / 2)>>10, 0));
  App_WrCoCmd_Buffer(phost, BITMAP_SIZE_H(((dc(e->w) >> 4) + 1)>>9, ((dc(e->h) >> 4) + 1)>>9));
#endif
}

static void draw(Gpu_Hal_Context_t *phost, const struct element *e, int32_t x)
{
  setup_element(phost, e);
  App_WrCoCmd_Buffer(phost, VERTEX2F(dc(e->x0) + x, YOFFSET + dc(e->y0)));
}

void App_Show_Logo(Gpu_Hal_Context_t *phost)
{
#if defined (BT815_ENABLE)
	Gpu_CoCmd_Logo(phost);
#else
    uint16_t ff,frame,i;
    int32_t syo ;

    scaling = FX16(1) * DispWidth / 800;
    YOFFSET = ((DispHeight << 4) - dc(242)) / 2;
    syo = YOFFSET * (FX16(800 / 16) / DispWidth);
    
    #if defined(MSVC_PLATFORM) || defined(MSVC_FT800EMU)
        Gpu_Hal_WrCmd32(phost,CMD_INFLATE);
        Gpu_Hal_WrCmd32(phost,0);
        Gpu_Hal_WrCmdBuf(phost, logo_graphics, sizeof(logo_graphics));
    #else
        Gpu_CoCmd_Inflate(phost, 0);
        Gpu_Hal_WrCmdBufFromFlash(phost, logo_graphics, sizeof(logo_graphics));
    #endif
        App_Flush_Co_Buffer(phost);
        Gpu_Hal_WaitCmdfifo_empty(phost);
    
    for (frame = 0, ff = 0; frame < 120; frame++, ff += 550) {
        Gpu_CoCmd_Dlstart(phost);
        App_WrCoCmd_Buffer(phost, CLEAR_COLOR_RGB(255,255,255));
        App_WrCoCmd_Buffer(phost, CLEAR(1, 1, 1));
        App_WrCoCmd_Buffer(phost, BITMAP_HANDLE(LOGO_ICON_HANDLE));
        App_WrCoCmd_Buffer(phost, BEGIN(BITMAPS));

        Gpu_CoCmd_LoadIdentity(phost);
        Gpu_CoCmd_Scale(phost, scaling, scaling);
        Gpu_CoCmd_SetMatrix(phost);

        App_WrCoCmd_Buffer(phost, COLOR_RGB( 35,  31,  32)); // gray
        for (i = 0; i < 10; i++) {
            if (i == 9)
                App_WrCoCmd_Buffer(phost, COLOR_RGB(237,  28,  36)); // red
            draw(phost, &el[i], tween((ff - FX16(.1)) * 3 - FX16(.1) * i, dc(-130), 0));
        }

        App_WrCoCmd_Buffer(phost, COLOR_RGB( 56, 113, 193)); // blue
        App_WrCoCmd_Buffer(phost, SAVE_CONTEXT());
        setup_element(phost, &el[10]);
        App_WrCoCmd_Buffer(phost, BITMAP_SIZE(BILINEAR, BORDER, BORDER, 0, 0));
        Gpu_CoCmd_LoadIdentity(phost);
        Gpu_CoCmd_Scale(phost, scaling, scaling);
        Gpu_CoCmd_Translate(phost, (int32_t)el[10].x0 << 16, ((int32_t)el[10].y0 << 16) + syo + FX16(60));
        Gpu_CoCmd_Rotate(phost, -tween((ff - FX16(.1)) * 2, 0x4000, 0x0000));
        Gpu_CoCmd_Translate(phost, FX16(0), FX16(-60));
        Gpu_CoCmd_SetMatrix(phost);
        App_WrCoCmd_Buffer(phost, VERTEX2F(0, 0));
        App_WrCoCmd_Buffer(phost, RESTORE_CONTEXT());

        draw(phost, &el[11], tween((ff - FX16(.4)) * 3, dc(210), 0));

        App_WrCoCmd_Buffer(phost, COLOR_RGB( 35,  31,  32)); // gray
        App_WrCoCmd_Buffer(phost, COLOR_A(tween((ff - FX16(.75)) * 4, 0, 255)));
        draw(phost, &el[12], 0);

        App_WrCoCmd_Buffer(phost, DISPLAY());

        Gpu_CoCmd_Swap(phost);
        App_Flush_Co_Buffer(phost);
        Gpu_Hal_WaitCmdfifo_empty(phost);
    }
#endif
    /* Show dummy clear screen to clean up logo after animation */
    Gpu_CoCmd_Dlstart(phost);
    Gpu_ClearScreen(phost);
    App_Flush_Co_Buffer(phost);
    Gpu_Hal_WaitCmdfifo_empty(phost);
    Gpu_Hal_Sleep(20);
}
#else
#define LOGO_HEIGHT 80 
#define LOGO_WIDTH 264 
#define LOGO_STRIDE 528
#define LOGO_FMT   ARGB4
PROGMEM prog_uchar8_t logo[SIZE_LOGO] = {//bridgetek 6703 bytes
    //logo
    120,156,237,93,205,78,35,201,178,230,216,189,31,117,122,214,231,200,105,214,247,72,152,39,192,60,1,102,125,71,2,150,173,123,36,232,221,100,222,35,1,203,68,71,194,60,1,244,19,0,251,43,97,246,35,1,79,128,121,2,170,214,211,55,50,242,39,34,178,12,216,204,12,61,67,59,75,24,168,159,172,204,140,47,190,248,201,172,242,210,210,124,69,183,244,150,62,213,106,206,203,22,229,93,22,68,195,35,108,103,221,5,34,190,251,162,149,222,208,21,108,30,17,167,106,129,136,239,188,68,110,8,91,165,207,165,213,208,90,111,168,173,133,37,249,126,138,222,85,143,42,226,1,127,159,37,233,3,115,236,171,123,228,142,5,30,190,155,162,219,250,64,48,196,113,144,62,160,225,34,239,93,224,225,187,42,122,23,253,7,207,4,199,221,191,249,61,93,173,198,25,13,247,90,127,235,22,46,202,31,91,32,166,88,35,41,235,14,32,194,75,126,20,185,65,235,49,227,140,5,30,222,121,233,180,33,166,120,212,87,12,17,109,189,15,91,178,20,87,194,130,84,186,243,237,218,186,40,127,116,233,182,48,194,244,178,190,97,136,104,233,22,254,238,100,52,84,209,138,236,133,35,139,242,62,75,119,131,105,255,149,94,17,199,18,55,84,57,226,216,208,127,251,86,45,93,148,63,190,124,84,106,66,242,46,172,134,66,191,161,98,120,217,90,112,195,123,47,221,181,32,113,159,109,80,254,175,221,176,95,75,110,128,79,181,64,195,119,80,192,83,216,80,247,153,1,82,76,209,105,120,145,11,52,188,235,242,49,75,23,163,205,32,247,99,213,198,61,26,208,64,62,195,227,2,13,239,187,116,91,106,168,239,213,26,201,88,15,61,55,132,236,147,230,217,167,16,85,108,176,51,85,183,61,235,125,76,207,244,77,223,245,205,170,89,183,3,179,110,6,110,213,12,96,223,58,108,126,239,178,89,254,52,115,109,79,220,227,71,168,49,212,234,235,93,53,171,51,92,209,143,215,248,79,127,205,111,108,195,159,161,152,1,142,106,232,145,31,221,57,250,4,22,34,100,150,56,34,116,195,82,164,121,206,45,113,214,120,246,252,131,217,54,149,169,97,171,92,29,255,242,191,249,86,187,59,243,5,122,241,227,236,173,47,238,49,76,53,153,112,143,139,23,175,248,209,214,134,111,149,235,189,246,238,127,158,146,199,22,63,93,101,102,236,83,47,100,159,130,159,56,209,107,242,40,128,34,162,193,251,150,136,154,33,69,152,224,99,222,204,147,143,114,67,144,55,201,42,200,191,142,251,42,91,103,25,86,128,137,15,179,214,42,139,25,90,170,197,127,94,191,120,5,224,193,197,246,96,75,30,94,143,198,63,79,241,99,105,189,222,165,113,152,17,15,104,25,200,26,8,68,196,8,147,199,152,91,138,115,195,205,124,249,106,179,25,229,196,144,203,254,151,140,49,154,189,239,226,30,67,86,155,255,28,191,120,69,47,178,2,93,243,62,240,192,245,174,122,217,110,134,162,87,80,170,83,230,35,0,13,231,58,207,118,123,134,224,17,38,114,195,227,124,243,221,102,211,86,209,86,132,223,100,53,106,198,14,169,15,175,202,131,135,123,48,60,188,204,15,203,225,108,27,177,105,31,94,119,231,215,23,243,33,108,159,62,252,220,49,237,79,237,215,114,35,47,78,88,64,248,89,158,245,74,129,8,63,163,29,98,10,68,67,198,66,211,111,160,107,102,199,195,182,96,131,170,64,48,252,101,249,158,237,249,70,32,222,99,147,249,14,245,140,246,34,99,52,92,105,223,216,127,128,54,143,205,181,185,179,240,3,191,175,205,193,239,80,103,64,56,141,246,204,120,88,90,82,61,244,3,188,85,56,206,243,20,23,209,78,36,91,193,98,10,165,209,199,76,71,103,183,23,219,209,103,152,98,53,28,73,209,123,18,222,159,56,153,167,255,249,30,195,92,247,236,252,144,124,174,208,134,201,91,219,139,168,39,164,23,103,191,67,157,220,27,155,193,127,0,249,178,149,110,81,223,143,131,119,32,185,1,55,54,107,165,1,61,42,177,70,5,246,101,118,126,96,190,127,28,253,137,57,48,135,224,61,142,147,70,39,92,216,25,52,123,234,61,124,244,184,238,214,49,222,92,245,159,47,94,209,65,44,196,209,115,223,192,127,48,123,73,15,162,70,95,254,246,58,29,199,151,255,124,6,15,42,197,20,187,108,159,214,187,145,27,52,139,48,195,207,150,64,131,204,84,62,206,190,202,214,238,56,222,66,175,147,49,26,52,109,123,105,101,15,106,55,126,155,60,0,242,3,89,218,202,76,220,91,251,15,219,174,226,152,252,61,240,192,250,83,63,111,47,252,140,118,246,18,119,203,60,99,183,163,152,188,85,200,69,102,185,104,178,20,201,171,152,135,31,118,138,88,2,226,202,124,108,88,180,191,54,183,111,227,215,217,229,116,231,136,200,183,183,23,59,78,70,91,191,15,30,42,142,115,251,100,124,161,54,10,75,192,228,41,214,69,54,230,48,209,207,16,220,48,103,188,25,253,73,155,198,158,101,139,204,122,30,141,132,139,187,55,227,7,25,247,190,125,124,177,39,116,228,119,193,131,21,140,99,171,163,39,249,33,75,60,174,138,84,217,106,160,223,64,243,151,165,165,64,110,80,143,60,6,197,28,214,204,252,112,180,157,37,158,36,32,241,80,29,85,44,95,117,247,54,114,193,252,67,230,45,204,77,189,117,124,177,195,16,89,147,21,253,77,117,6,155,91,103,214,123,178,79,74,169,179,204,248,184,74,54,236,199,152,162,42,184,129,91,138,155,156,177,230,252,48,79,188,73,253,78,158,91,214,4,51,32,175,34,246,224,154,240,96,6,246,196,156,152,17,252,120,255,243,4,98,196,14,212,54,198,30,79,0,57,163,212,95,240,32,71,112,198,97,60,251,196,238,136,22,180,129,163,46,128,1,2,222,78,204,234,81,219,117,40,198,141,89,16,161,75,224,111,126,6,219,21,44,201,248,232,179,63,10,156,226,91,2,119,114,7,198,111,133,246,193,125,214,161,197,151,120,93,170,123,2,119,62,128,246,137,236,2,244,98,228,46,211,184,132,220,170,157,216,195,216,211,85,81,103,31,90,114,1,117,78,136,67,237,5,156,57,108,90,184,196,56,57,203,211,139,163,115,128,53,251,79,191,225,232,228,248,129,71,152,194,51,40,45,5,174,126,168,68,252,73,168,152,59,222,100,209,21,241,67,223,49,159,14,199,133,105,137,59,97,125,243,127,13,204,153,139,179,32,46,100,179,238,194,56,71,31,37,176,164,63,194,242,147,230,3,140,38,101,204,195,89,123,102,89,180,168,136,47,224,232,53,179,195,53,218,147,33,176,153,204,169,14,184,220,236,166,185,99,60,87,102,10,199,166,255,41,99,2,90,132,189,117,73,67,128,33,115,198,189,54,67,214,142,47,209,19,192,26,29,213,91,35,82,183,63,9,76,48,116,135,179,49,206,178,35,97,151,106,243,57,202,183,21,17,177,175,115,132,169,178,165,192,159,237,169,217,167,170,129,135,57,242,15,118,155,36,238,138,72,59,249,15,14,229,136,178,102,249,7,208,181,58,230,87,194,118,200,228,145,236,36,98,61,228,163,92,66,149,200,87,131,118,213,52,30,54,233,207,136,249,44,120,141,205,35,11,58,121,41,44,92,26,199,67,236,67,150,143,229,120,216,33,57,165,156,39,175,3,91,187,153,207,190,36,198,52,52,223,16,115,48,38,178,27,216,180,59,134,1,134,103,71,168,188,252,153,249,91,12,171,248,151,207,177,1,211,221,57,190,151,89,17,68,0,229,27,120,132,89,172,111,80,180,250,161,42,124,135,121,243,81,59,100,213,76,208,9,178,23,123,66,7,107,26,11,60,122,72,210,112,21,103,146,152,233,70,140,224,153,67,142,20,56,154,179,24,166,237,238,210,24,58,25,201,212,142,203,155,197,155,102,221,77,211,240,170,104,105,109,214,243,21,61,224,15,118,127,142,35,199,175,233,76,195,67,209,175,152,163,5,94,251,34,238,15,154,17,241,82,57,126,175,207,54,243,142,165,17,8,248,234,49,159,61,105,209,152,219,174,143,42,91,138,115,134,133,105,153,233,116,164,82,194,115,136,140,242,26,255,33,97,98,12,227,231,51,72,145,57,8,189,78,204,50,102,62,160,241,21,242,193,235,70,5,30,194,217,132,135,158,161,60,88,205,228,204,207,70,123,192,248,97,196,36,200,239,21,101,151,176,232,214,89,75,177,119,249,44,118,182,227,200,59,140,231,95,136,220,67,69,188,131,123,2,30,150,169,141,52,235,150,163,52,26,17,22,161,23,188,84,123,123,225,4,170,128,133,247,154,50,234,42,149,208,80,77,65,3,250,13,170,100,4,185,205,140,7,63,223,45,101,201,178,104,92,206,254,232,103,129,164,145,147,71,121,79,227,72,71,60,108,134,61,54,201,144,240,16,253,4,226,112,49,163,198,173,61,141,234,88,178,141,144,113,226,255,154,248,1,236,203,53,59,230,207,155,0,166,188,247,54,145,24,118,227,112,23,239,63,200,59,216,132,90,47,49,244,31,204,142,99,87,162,206,95,0,238,70,222,134,20,154,65,184,44,121,108,25,244,238,33,176,74,198,100,35,6,237,202,53,211,62,250,220,104,250,13,74,102,160,56,50,230,183,23,201,178,115,185,86,124,182,25,228,117,107,118,172,244,194,15,229,120,192,207,200,238,193,121,92,150,145,31,82,77,81,119,201,99,237,21,136,243,247,242,190,246,173,224,128,154,143,147,185,107,200,97,21,234,249,44,25,131,225,161,147,229,158,238,17,61,194,104,15,233,186,152,245,130,186,70,128,33,198,62,206,235,121,136,142,70,193,11,116,103,236,58,95,203,69,200,204,176,153,250,132,255,60,7,216,208,176,158,221,20,250,48,101,37,0,203,62,77,227,6,157,230,41,104,203,171,98,200,247,156,35,63,105,119,50,58,107,209,214,96,231,194,231,5,120,157,205,232,137,227,193,159,143,126,168,25,136,186,208,255,244,254,164,136,32,114,175,205,114,225,243,71,15,5,172,85,108,85,244,215,88,62,202,76,242,232,5,59,188,202,112,194,89,101,144,238,81,244,170,74,243,39,44,247,146,112,151,125,185,104,45,171,232,69,54,242,81,140,115,194,103,204,234,122,141,23,220,194,88,181,64,3,216,11,31,91,101,77,241,125,45,230,143,99,212,89,69,123,128,217,39,113,244,70,96,161,42,50,81,20,125,206,227,63,112,28,200,209,201,156,97,39,148,77,16,120,96,168,137,178,95,53,228,137,72,255,129,124,86,110,47,184,231,150,217,213,243,134,227,126,58,247,31,38,197,168,70,60,88,201,76,217,127,64,108,113,230,32,4,161,173,180,92,122,132,186,207,134,36,90,53,243,81,230,146,251,44,169,167,136,135,59,151,252,235,128,230,236,17,152,232,107,216,132,129,62,198,109,220,186,20,99,172,15,10,95,160,25,97,54,125,133,102,116,49,175,63,153,251,229,234,98,29,37,183,36,181,68,47,176,58,183,227,149,27,229,209,167,241,165,248,130,163,174,137,7,210,179,128,135,142,168,219,255,112,60,72,127,37,106,59,218,24,206,54,137,31,6,150,159,95,49,230,224,237,10,159,196,15,113,94,39,75,171,228,135,113,186,87,176,252,46,70,233,204,58,165,214,11,60,248,159,163,212,179,28,161,69,150,28,31,9,139,12,250,127,47,100,187,75,235,34,243,243,252,12,15,57,178,40,48,2,251,231,89,79,203,252,99,151,189,39,83,172,71,137,253,227,185,185,81,232,119,94,47,19,241,32,230,68,79,226,184,115,105,8,123,81,50,82,196,195,114,190,115,29,35,28,102,47,216,21,85,137,7,71,24,138,82,119,125,81,191,255,123,224,215,61,65,196,56,76,247,240,61,62,170,185,126,162,191,152,251,110,171,114,13,112,94,13,144,172,237,23,64,66,27,234,252,49,230,77,171,236,45,113,123,65,136,13,249,58,137,250,29,121,143,56,227,157,164,204,214,69,118,181,152,209,230,182,33,89,8,137,137,121,214,71,85,57,10,15,219,29,216,123,191,125,230,60,17,251,118,70,209,113,228,7,186,46,224,161,7,209,251,5,254,248,207,16,155,109,59,210,51,63,182,207,225,97,16,107,145,122,43,248,193,37,11,37,248,193,222,17,187,227,95,217,94,20,235,14,124,15,111,113,155,32,123,115,228,75,255,33,182,237,200,75,54,207,250,50,60,228,235,176,134,219,184,113,46,146,120,168,167,108,164,115,147,50,182,136,239,116,8,146,221,85,114,93,164,200,76,43,66,5,101,164,8,17,243,204,119,111,179,145,10,237,34,239,127,32,218,236,55,238,215,29,18,214,113,60,158,92,59,69,243,230,141,25,212,101,38,241,168,187,184,191,87,240,195,196,50,126,112,149,104,83,233,79,38,20,113,255,65,218,250,58,102,143,104,110,41,121,9,20,197,176,117,132,206,159,87,172,143,242,246,130,24,196,202,94,136,79,215,176,23,184,23,51,190,66,159,26,107,75,34,30,26,79,98,130,231,64,118,164,204,60,84,204,94,100,68,204,190,30,38,201,138,233,8,73,171,99,43,199,145,226,255,202,107,155,204,161,149,122,240,228,234,235,184,6,139,198,139,227,65,214,17,237,133,237,144,252,80,94,19,35,253,201,196,185,158,111,56,30,248,248,38,60,172,178,156,51,232,58,203,72,114,253,12,190,0,231,7,178,237,66,75,226,241,107,145,133,228,214,72,122,49,210,127,160,168,40,33,157,176,253,96,11,43,143,246,130,180,156,229,223,213,90,225,89,60,210,19,190,90,190,93,44,28,155,121,118,216,113,219,30,122,67,249,234,15,209,151,231,189,163,217,156,3,57,30,246,105,60,108,70,45,74,231,147,189,88,45,198,50,106,59,88,225,160,67,169,77,19,233,63,8,134,104,250,147,140,105,200,195,117,92,31,171,204,106,130,255,108,105,47,168,127,5,63,56,17,111,202,89,140,56,15,150,242,15,133,255,112,196,241,40,153,107,40,239,145,121,32,88,128,123,181,193,142,245,245,164,193,12,101,84,241,152,217,98,190,249,77,105,241,152,165,140,179,208,21,235,53,161,253,32,242,74,234,213,211,246,98,147,141,148,223,88,124,193,162,61,38,69,183,44,89,201,114,60,60,36,73,6,77,119,9,15,119,130,29,200,159,28,48,29,174,24,10,26,26,234,184,215,186,205,236,136,151,108,51,190,144,30,142,244,130,248,190,2,15,108,246,234,172,88,189,94,96,14,252,132,227,66,194,107,93,178,26,107,69,188,89,53,254,163,191,230,176,23,41,67,156,71,134,245,252,232,161,224,213,138,214,157,155,131,44,203,160,251,207,249,15,21,207,68,83,44,239,122,114,252,108,244,14,153,255,144,238,207,249,129,228,90,250,15,252,154,196,15,171,153,167,211,21,123,208,162,77,124,146,114,24,255,90,135,79,63,99,78,222,242,142,108,65,35,255,192,249,193,247,126,12,181,109,198,26,125,77,169,230,33,249,168,114,70,4,174,73,126,85,226,193,135,70,254,33,101,156,146,246,87,226,217,172,21,61,225,30,4,123,46,71,162,226,53,254,36,73,140,245,28,99,59,206,230,85,154,243,129,222,29,8,173,136,249,135,169,247,72,54,169,138,218,33,231,47,210,232,132,154,114,62,42,213,29,229,78,158,30,229,255,132,71,131,252,192,180,221,50,60,136,8,163,158,229,185,168,152,167,163,190,79,207,63,80,173,95,166,215,35,174,73,154,151,198,173,111,39,108,252,252,231,78,121,77,145,117,242,79,235,113,171,225,253,136,105,118,162,240,51,213,188,243,23,209,143,54,194,219,115,109,251,144,101,24,172,98,109,51,63,56,244,39,221,44,120,216,36,158,182,211,227,77,98,137,28,95,36,223,33,242,54,249,147,69,62,216,114,255,161,166,213,71,86,242,3,247,18,102,193,67,158,221,13,188,105,167,241,3,103,144,25,158,101,164,181,25,177,61,125,54,87,27,144,63,101,85,94,119,37,115,68,154,141,88,83,220,106,148,158,229,99,99,78,107,158,231,245,182,153,246,132,30,178,248,130,75,43,102,32,184,255,80,101,28,249,223,79,227,129,158,109,9,191,51,63,216,158,208,177,42,233,46,216,145,232,141,133,149,86,238,65,240,131,176,213,204,127,144,119,161,120,83,122,196,153,31,160,127,219,102,199,110,251,79,179,141,27,161,238,115,182,46,21,102,170,202,252,195,53,217,50,28,191,220,123,96,195,80,23,212,233,182,237,78,194,43,241,131,77,109,95,110,100,234,234,105,235,83,197,147,119,1,19,210,106,220,51,123,50,53,222,156,195,159,28,50,95,61,180,44,51,163,219,100,109,79,89,107,90,69,116,16,121,62,29,121,206,159,76,35,23,70,132,199,23,114,60,164,189,32,15,176,204,79,242,171,146,116,111,211,172,144,11,217,51,142,135,218,230,217,16,75,249,234,30,235,87,148,80,190,75,90,55,150,142,150,246,226,50,107,72,225,11,198,217,114,194,31,139,55,93,242,146,130,30,245,253,108,71,182,111,225,183,88,255,160,90,121,61,204,77,246,34,188,37,184,23,86,99,101,26,71,4,158,136,239,182,158,107,62,75,172,17,242,94,141,95,53,122,224,215,255,8,15,72,88,235,144,143,178,28,75,135,79,222,99,40,240,86,23,235,97,228,177,3,159,245,181,67,118,199,48,86,98,254,130,229,196,19,130,62,36,156,4,185,31,17,63,244,242,88,167,62,150,150,36,72,199,255,197,231,47,130,173,72,26,113,43,179,69,246,44,35,85,176,42,180,227,142,86,75,224,49,158,159,164,126,198,177,116,95,242,255,97,187,160,251,232,150,222,207,79,100,245,195,26,168,108,9,228,155,65,200,106,76,155,223,186,159,61,255,16,125,92,222,202,248,67,235,213,88,76,198,71,236,48,106,87,186,242,105,123,17,125,179,163,42,114,68,99,61,12,187,179,95,99,157,116,38,63,223,45,243,15,100,223,80,134,119,102,7,152,76,172,108,53,217,19,89,74,51,76,28,185,23,192,25,29,156,109,38,175,213,31,225,119,217,201,154,156,61,74,179,7,62,244,118,92,35,124,98,243,204,13,142,148,95,209,219,1,236,237,25,194,72,216,178,143,24,71,129,176,2,204,102,55,133,182,249,159,56,194,202,71,155,94,186,123,33,198,236,246,26,79,253,179,12,85,180,26,211,60,203,199,121,242,147,110,147,107,39,91,113,192,117,150,230,172,46,197,252,69,240,35,19,126,158,243,31,120,159,185,189,248,49,107,46,254,78,49,127,241,174,154,90,228,201,199,12,161,116,70,45,120,159,225,97,137,214,59,81,173,132,114,126,21,199,195,208,36,31,88,174,156,187,141,120,216,180,172,133,140,39,203,251,176,245,81,92,175,176,167,203,44,146,74,43,211,171,188,222,254,56,207,67,236,134,103,44,148,86,55,82,230,170,140,53,154,153,135,240,123,190,231,179,216,232,91,62,74,181,169,139,158,178,25,111,62,127,129,231,60,159,143,170,217,217,196,15,157,152,69,146,216,227,163,154,254,34,123,113,200,89,44,75,54,98,211,209,232,210,58,181,29,214,7,174,241,5,66,68,38,110,144,81,35,207,78,107,234,122,230,65,100,155,57,207,72,125,34,70,165,189,225,119,31,237,203,165,225,94,12,206,99,160,165,136,254,2,174,113,73,111,152,12,25,75,142,9,230,89,170,149,167,86,68,204,241,60,111,94,7,36,70,172,50,210,58,134,94,92,115,239,55,231,171,211,72,61,151,127,224,90,84,243,85,97,225,41,142,242,206,166,98,171,28,194,70,158,94,159,114,91,71,133,54,138,117,181,196,15,29,246,12,78,205,87,66,179,213,48,194,55,74,72,165,39,48,242,245,241,137,69,215,22,43,244,100,6,157,107,215,136,252,1,203,231,82,35,63,44,209,138,36,106,225,170,26,21,204,127,156,234,200,177,70,138,39,238,27,177,70,211,139,152,135,31,226,250,73,203,185,128,121,193,241,237,49,13,52,228,120,115,118,126,160,186,249,243,23,189,32,21,146,144,151,194,209,129,215,26,182,242,110,194,51,119,40,137,20,69,164,90,211,154,75,66,212,58,187,162,159,51,239,220,162,48,59,3,190,99,225,15,251,168,200,86,226,42,31,121,230,89,72,176,117,103,98,189,69,97,193,16,33,226,249,231,204,33,9,179,203,52,2,140,183,42,115,232,191,207,130,229,16,142,53,243,101,181,183,26,41,182,12,126,68,25,107,164,152,147,214,217,205,131,7,201,98,137,111,43,97,163,199,48,58,69,100,108,15,13,63,227,185,120,115,40,112,83,172,26,133,154,31,152,231,232,183,81,120,106,75,72,143,175,244,111,67,171,111,5,151,236,216,182,189,21,253,168,237,186,184,203,50,174,167,102,186,206,162,69,176,96,222,27,44,90,237,159,35,124,136,209,43,141,204,221,167,142,56,227,86,180,146,158,35,169,112,5,174,212,32,54,198,152,203,75,207,51,94,135,61,49,243,5,40,2,123,177,203,208,80,176,61,203,89,166,136,163,244,44,57,183,204,245,62,16,219,137,239,68,196,79,183,25,255,11,111,162,28,184,117,124,3,101,103,218,83,221,128,235,1,230,253,67,150,126,240,244,219,12,240,204,184,57,255,89,188,15,4,238,176,99,78,156,95,63,243,5,188,131,117,63,138,225,221,147,14,219,98,253,103,209,2,227,219,189,3,76,177,157,222,77,153,231,47,146,71,60,40,174,104,227,123,48,247,220,200,156,1,218,224,199,250,251,249,185,140,229,167,222,14,5,119,25,90,31,121,135,43,70,230,208,238,252,252,161,56,163,239,91,111,191,64,235,47,225,156,208,135,61,104,93,115,5,242,32,142,172,223,134,118,61,63,255,179,26,71,125,104,163,28,208,159,220,69,105,142,196,147,22,91,34,31,193,99,141,102,134,234,145,173,160,251,14,191,15,197,222,10,155,84,155,245,151,175,249,51,23,228,136,99,134,134,14,174,181,142,235,164,188,103,169,42,38,245,39,50,84,225,249,172,247,252,45,140,160,231,189,252,230,161,30,108,81,95,253,186,230,108,7,166,240,195,95,175,232,86,183,68,3,72,183,155,56,98,69,223,138,153,204,233,158,229,187,231,7,96,119,242,3,252,22,86,208,180,193,214,167,108,64,96,137,191,56,63,240,162,58,250,138,36,223,221,237,38,68,112,171,209,68,68,90,49,115,255,174,249,161,147,178,36,49,35,116,129,43,13,78,152,111,31,50,161,47,190,181,236,175,82,116,79,124,59,26,203,63,139,39,121,195,54,125,94,227,125,227,225,65,196,252,20,167,197,217,181,24,233,189,131,55,160,135,18,214,221,39,223,80,201,39,253,253,154,24,137,136,105,115,159,51,219,11,240,90,124,105,241,119,152,129,31,171,148,18,49,47,236,225,126,174,210,93,85,124,47,48,212,163,216,62,213,130,127,249,247,53,168,110,11,63,101,173,43,221,214,210,220,5,34,131,34,214,231,145,114,252,171,241,190,72,236,3,52,146,158,77,193,61,234,35,239,89,11,116,81,23,237,134,43,160,47,109,113,21,246,69,244,190,147,222,54,159,246,40,245,81,124,111,149,110,119,53,191,87,168,29,36,150,107,238,182,60,175,203,154,227,29,119,153,188,183,138,163,90,223,42,190,226,161,156,215,152,204,195,15,31,87,98,45,251,185,134,78,140,110,207,233,219,226,161,61,19,54,142,129,133,174,104,213,141,74,79,25,231,182,126,212,156,187,148,207,190,110,104,191,143,190,235,105,109,222,216,56,21,124,19,140,204,158,21,121,35,51,229,221,76,153,59,41,215,119,30,247,196,111,36,3,73,28,99,6,39,123,245,122,5,254,91,129,173,162,111,45,211,62,119,184,226,91,207,234,14,242,186,97,111,15,221,135,255,164,142,157,203,113,94,10,223,188,93,209,121,202,143,72,31,182,170,209,118,202,71,136,39,253,131,54,245,122,5,71,148,126,196,205,28,252,208,7,143,117,77,29,131,44,219,249,222,183,48,34,27,92,158,208,26,38,55,248,59,28,207,210,135,86,194,136,233,83,117,149,247,8,60,96,54,254,216,239,75,24,251,216,210,87,48,66,30,189,5,222,95,46,230,67,122,86,208,17,22,226,95,24,93,60,76,139,45,240,109,158,192,166,12,215,87,254,187,200,128,17,146,244,253,236,192,30,244,37,247,85,15,96,92,214,244,10,103,97,125,172,252,147,115,43,132,7,212,160,83,196,55,141,200,62,71,7,242,122,165,135,56,14,73,250,29,117,175,207,176,238,56,78,248,124,222,22,212,125,211,108,125,224,8,197,223,93,239,229,188,155,99,13,49,223,173,30,249,183,238,73,230,122,190,96,47,180,127,163,126,55,142,147,110,67,221,251,184,146,147,56,163,196,195,126,64,5,235,253,61,92,167,152,246,131,236,149,196,195,173,199,94,30,105,143,186,253,143,158,193,95,99,49,62,128,7,121,45,178,154,201,78,76,154,111,18,75,5,223,234,200,52,5,188,52,177,62,13,144,225,87,48,123,14,207,252,160,3,63,8,60,192,127,99,206,15,40,199,221,240,201,122,44,240,128,250,163,195,104,231,171,252,204,101,11,134,32,106,34,88,212,145,190,0,45,97,28,194,106,104,241,182,171,126,68,192,110,153,161,82,105,85,221,198,180,90,94,42,200,133,251,94,87,19,242,224,190,94,255,215,124,164,155,207,218,229,49,75,96,61,125,163,78,89,239,39,208,63,208,164,228,31,64,251,64,31,216,241,43,117,47,70,195,235,212,49,142,245,171,99,99,136,60,55,205,182,249,140,235,119,124,166,114,253,249,183,18,34,63,115,60,156,67,191,183,152,85,19,114,199,51,86,144,215,246,5,30,78,97,243,223,57,66,120,208,168,215,94,131,242,234,82,184,230,182,129,7,133,35,144,172,147,66,105,246,224,30,244,198,168,33,183,95,207,244,100,192,178,11,41,67,213,231,209,39,162,226,21,25,24,108,161,175,231,52,73,178,155,190,53,225,138,124,63,201,15,106,130,120,184,210,217,103,67,126,88,67,84,38,150,41,237,197,88,157,251,113,77,246,66,181,61,255,160,174,101,150,249,163,11,126,171,45,155,81,192,40,238,94,147,141,243,210,23,99,232,237,5,156,113,207,199,22,218,124,170,177,47,121,79,144,108,137,135,251,46,183,23,222,162,42,189,33,52,98,162,118,193,98,60,178,171,188,46,222,188,196,238,222,131,140,153,199,224,53,108,231,246,79,242,123,98,170,210,143,152,173,68,70,28,178,118,122,73,141,60,99,176,179,164,189,136,252,160,25,63,40,111,47,118,201,30,0,11,78,36,63,128,6,93,209,202,111,100,215,125,124,247,209,219,225,97,91,122,218,192,205,167,60,178,210,13,126,64,15,15,237,133,226,246,226,20,181,159,227,1,215,172,20,246,226,190,224,135,132,135,152,23,65,6,221,3,211,126,43,124,204,115,26,213,39,122,209,41,214,198,221,51,95,247,217,121,141,89,138,215,128,174,134,207,236,65,35,147,239,227,216,145,55,80,224,65,237,119,91,193,139,96,189,239,96,20,66,190,153,228,135,43,220,35,81,119,160,91,111,138,135,13,249,36,2,140,253,185,56,142,252,0,253,56,200,168,246,72,40,253,135,17,224,193,123,66,47,249,15,18,15,97,156,165,255,224,175,106,193,152,177,53,180,234,74,182,105,74,47,20,218,47,146,184,176,46,104,187,120,172,49,153,207,106,40,255,93,175,167,128,246,251,148,9,136,120,128,122,165,255,192,236,197,61,96,250,92,140,209,6,218,217,27,234,139,46,249,97,12,61,247,216,37,175,245,2,234,28,65,228,92,188,35,231,143,43,168,163,210,159,244,62,204,49,151,144,151,7,249,74,79,248,147,167,161,47,121,79,11,174,186,65,175,130,206,242,50,27,121,15,41,254,15,81,36,216,134,177,247,187,195,30,213,10,247,87,34,167,168,206,95,194,195,210,82,92,61,53,5,13,177,205,19,198,30,232,89,206,238,179,131,220,46,96,59,102,156,211,134,255,182,186,30,133,25,183,80,227,41,243,39,189,253,188,208,91,138,101,82,0,49,30,35,196,40,126,254,133,227,101,31,81,117,46,190,223,3,112,168,142,95,19,95,188,174,128,118,158,9,157,221,71,143,242,66,228,68,192,191,103,113,81,31,35,66,104,169,202,227,3,54,127,23,251,194,120,29,206,241,53,237,50,143,107,136,209,236,85,26,3,144,225,150,199,154,98,252,141,111,149,188,2,54,226,57,186,3,242,38,158,233,73,200,80,113,75,241,131,250,73,255,75,253,128,45,140,209,39,155,235,122,199,249,234,69,241,5,51,84,28,13,255,234,252,170,127,213,255,238,33,34,216,91,8,67,180,177,192,195,119,85,244,79,250,171,254,170,126,237,253,170,255,55,238,89,17,43,111,223,241,124,247,162,200,162,127,208,255,13,204,240,107,231,107,239,171,246,136,248,31,29,56,130,158,215,120,197,156,192,162,252,85,139,254,169,247,85,125,213,184,41,143,136,255,83,255,136,71,52,250,184,143,250,93,207,119,47,138,44,157,127,3,59,0,14,84,64,197,47,234,159,116,12,98,152,21,92,117,183,192,195,119,83,244,223,245,127,122,145,29,122,191,232,255,250,214,237,89,148,111,93,122,63,232,255,160,189,248,69,255,243,229,179,23,229,253,23,237,17,177,64,195,187,45,255,15,219,213,59,101,
};


void App_Show_Logo(Gpu_Hal_Context_t *phost)
{

    uint32_t x, y, w, h;
    uint32_t step;
    #define TOTAL_STEP 8
    const float SCALE_INIT = 1.7;
    float scale = 1;

#if defined(MSVC_PLATFORM) || defined(MSVC_FT800EMU)
    Gpu_Hal_WrCmd32(phost,CMD_INFLATE);
    Gpu_Hal_WrCmd32(phost,0);
    Gpu_Hal_WrCmdBuf(phost, logo, sizeof(logo));
#else
    Gpu_CoCmd_Inflate(phost, 0);
    Gpu_Hal_WrCmdBufFromFlash(phost, logo, sizeof(logo));
#endif
    //Starting at SCALE_INIT times of orginial logo
    //Display animation: scale logo from SCALE_INIT to X1 size
    scale = SCALE_INIT;
    for (step = 1; step <= TOTAL_STEP; step++) {
        scale -= (SCALE_INIT-1)/TOTAL_STEP;
        x = (uint32_t)(DispWidth-scale*LOGO_WIDTH)/2;
        y = (uint32_t)(DispHeight-scale*LOGO_HEIGHT)/2;
        w = (uint32_t)(scale*LOGO_WIDTH);
        h = (uint32_t)(scale*LOGO_HEIGHT);

        Gpu_CoCmd_Dlstart(phost);
        App_WrCoCmd_Buffer(phost, CLEAR_TAG(1));
        App_WrCoCmd_Buffer(phost, CLEAR_COLOR_RGB(255,255,255));
        App_WrCoCmd_Buffer(phost, CLEAR(1, 1, 1));
        
        //Display Logo
        //App_WrCoCmd_Buffer(phost, COLOR_MASK(0, 0, 0, 0));
        App_WrCoCmd_Buffer(phost, COLOR_A(255));
        App_WrCoCmd_Buffer(phost, COLOR_RGB(255,255,255));
        App_WrCoCmd_Buffer(phost, BITMAP_HANDLE(LOGO_ICON_HANDLE));
        Gpu_CoCmd_LoadIdentity(phost);
        Gpu_CoCmd_Scale(phost, scale*65536L,scale*65536L);
        Gpu_CoCmd_SetMatrix(phost );
        //#ifdef FT81X_ENABLE
        //Gpu_CoCmd_SetBitmap(phost, 0, LOGO_FMT, w, h);
        //#else
        App_WrCoCmd_Buffer(phost,BITMAP_SOURCE(0));
        App_WrCoCmd_Buffer(phost,BITMAP_LAYOUT(LOGO_FMT, LOGO_STRIDE,LOGO_HEIGHT));
        App_WrCoCmd_Buffer(phost,BITMAP_SIZE(NEAREST, BORDER, BORDER, w, h));    
        //#endif
        App_WrCoCmd_Buffer(phost,BEGIN(BITMAPS));
        App_WrCoCmd_Buffer(phost, BITMAP_HANDLE(LOGO_ICON_HANDLE));
        
        App_WrCoCmd_Buffer(phost,VERTEX2F(x<<4 , y<<4));

        //DONE
        App_WrCoCmd_Buffer(phost,TAG_MASK(0));
        App_WrCoCmd_Buffer(phost,DISPLAY());
        App_Flush_Co_Buffer(phost);
        /*Swap the frame directly*/
        Gpu_Hal_Wr8(phost,REG_DLSWAP,DLSWAP_FRAME);
        Gpu_Hal_WaitCmdfifo_empty(phost);
        Gpu_Hal_Sleep(150);
    }
    /* Show dummy clear screen to clean up logo after animation */
    Gpu_CoCmd_Dlstart(phost);
    Gpu_ClearScreen(phost);
	App_Flush_Co_Buffer(phost);
    Gpu_Hal_WaitCmdfifo_empty(phost);
    Gpu_Hal_Sleep(150);
}

#endif
/******************** END DRAW BRIDGETEK LOGO ****************************/

/*************************************************************************************************
                        APP FUNCTIONALITIES
*************************************************************************************************/

void App_Common_Init(Gpu_Hal_Context_t *phost)
{

#if ( defined(FT900_PLATFORM) || defined(FT93X_PLATFORM) )
    FT9XX_Config();
#endif

    Gpu_HalInit_t halinit;
    
    halinit.TotalChannelNum = 1;

    
    Gpu_Hal_Init(&halinit);
    phost->hal_config.channel_no = 0;
    phost->hal_config.pdn_pin_no = FT800_PD_N;
    phost->hal_config.spi_cs_pin_no = FT800_SEL_PIN;

    #if defined(MSVC_PLATFORM) && defined(MSVC_PLATFORM_SPI_LIBMPSSE)
    phost->hal_config.spi_clockrate_khz = 12000; //in KHz
    #elif defined(MSVC_PLATFORM) && defined(MSVC_PLATFORM_SPI_LIBFT4222)
    #if defined(HAL_CLK_COMPUTE_TEST)
    FT4222_ClockRate selclk = 0;
    FT4222_SPIClock seldiv = 0;

    phost->hal_config.spi_clockrate_khz = 1;
    while (phost->hal_config.spi_clockrate_khz < 30001)
    {
        /*User requested SPI communication clock is in host->hal_config.spi_clockrate_khz
                Finding FT422 system clock and divider to provide this. If not nearest possible
                available communication clock will be chosen
                */
        if (!Gpu_Hal_FT4222_ComputeCLK(&host, &selclk, &seldiv))
        {
            printf("Requested clock %d KHz is not supported in FT4222 \n", phost->hal_config.spi_clockrate_khz);
            return 0;
        }
        phost->hal_config.spi_clockrate_khz += 1;
    }
    fflush(stdout);
    #else
    phost->hal_config.spi_clockrate_khz = 20000; //in KHz
    #endif
    #elif defined( ARDUINO_PLATFORM_SPI)
    phost->hal_config.spi_clockrate_khz = 4000; //in KHz
    #else /* FT900_PLATFORM or FT93X_PLATFORM */
    phost->hal_config.spi_clockrate_khz = 20000;
    #endif
    Gpu_Hal_Open(phost);
    BootupConfig(phost);

#if (defined (FT900_PLATFORM) || defined(FT900_PLATFORM) || defined(MSVC_PLATFORM))
    printf("\n reg_touch_rz =0x%x ", Gpu_Hal_Rd16(phost, REG_TOUCH_RZ));
    printf("\n reg_touch_rzthresh =0x%x ", Gpu_Hal_Rd32(phost, REG_TOUCH_RZTHRESH));
    printf("\n reg_touch_tag_xy=0x%x",Gpu_Hal_Rd32(phost, REG_TOUCH_TAG_XY));
    printf("\n reg_touch_tag=0x%x",Gpu_Hal_Rd32(phost, REG_TOUCH_TAG));
#endif

    Gpu_ClearScreen(phost);
    
    Gpu_Hal_Sleep(1000);//Show the booting up screen.


}

/**
* This function shows Bridgetek logo, Calibration and Welcome screen
* Display the application information in info array
* @Return: None
*/
void App_Common_Start(Gpu_Hal_Context_t *phost, char* info[])
{
    /* Screen Calibration*/
    App_Calibrate_Screen(phost);

    /* Show Bridgetek logo */
    App_Show_Logo(phost);

    /* Show welcome screen, waiting user to press Play button */
    App_Show_WelcomeScreen(phost,info);
}

void App_Common_Close(Gpu_Hal_Context_t *phost)
{
    /* Close all the opened handles */
    Gpu_Hal_Close(phost);
    Gpu_Hal_DeInit();
}


/*************************************************************************************************
                        Application utilities
*************************************************************************************************/
